(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.wasmoon = {}));
})(this, (function (exports) { 'use strict';

    exports.LuaReturn = void 0;
    (function (LuaReturn) {
        LuaReturn[LuaReturn["Ok"] = 0] = "Ok";
        LuaReturn[LuaReturn["Yield"] = 1] = "Yield";
        LuaReturn[LuaReturn["ErrorRun"] = 2] = "ErrorRun";
        LuaReturn[LuaReturn["ErrorSyntax"] = 3] = "ErrorSyntax";
        LuaReturn[LuaReturn["ErrorMem"] = 4] = "ErrorMem";
        LuaReturn[LuaReturn["ErrorErr"] = 5] = "ErrorErr";
        LuaReturn[LuaReturn["ErrorFile"] = 6] = "ErrorFile";
    })(exports.LuaReturn || (exports.LuaReturn = {}));
    const PointerSize = 4;
    const LUA_MULTRET = -1;
    const LUAI_MAXSTACK = 1000000;
    const LUA_REGISTRYINDEX = -LUAI_MAXSTACK - 1000;
    exports.LuaType = void 0;
    (function (LuaType) {
        LuaType[LuaType["None"] = -1] = "None";
        LuaType[LuaType["Nil"] = 0] = "Nil";
        LuaType[LuaType["Boolean"] = 1] = "Boolean";
        LuaType[LuaType["LightUserdata"] = 2] = "LightUserdata";
        LuaType[LuaType["Number"] = 3] = "Number";
        LuaType[LuaType["String"] = 4] = "String";
        LuaType[LuaType["Table"] = 5] = "Table";
        LuaType[LuaType["Function"] = 6] = "Function";
        LuaType[LuaType["Userdata"] = 7] = "Userdata";
        LuaType[LuaType["Thread"] = 8] = "Thread";
    })(exports.LuaType || (exports.LuaType = {}));
    exports.LuaEventCodes = void 0;
    (function (LuaEventCodes) {
        LuaEventCodes[LuaEventCodes["Call"] = 0] = "Call";
        LuaEventCodes[LuaEventCodes["Ret"] = 1] = "Ret";
        LuaEventCodes[LuaEventCodes["Line"] = 2] = "Line";
        LuaEventCodes[LuaEventCodes["Count"] = 3] = "Count";
        LuaEventCodes[LuaEventCodes["TailCall"] = 4] = "TailCall";
    })(exports.LuaEventCodes || (exports.LuaEventCodes = {}));
    exports.LuaEventMasks = void 0;
    (function (LuaEventMasks) {
        LuaEventMasks[LuaEventMasks["Call"] = 1] = "Call";
        LuaEventMasks[LuaEventMasks["Ret"] = 2] = "Ret";
        LuaEventMasks[LuaEventMasks["Line"] = 4] = "Line";
        LuaEventMasks[LuaEventMasks["Count"] = 8] = "Count";
    })(exports.LuaEventMasks || (exports.LuaEventMasks = {}));
    exports.LuaLibraries = void 0;
    (function (LuaLibraries) {
        LuaLibraries["Base"] = "_G";
        LuaLibraries["Coroutine"] = "coroutine";
        LuaLibraries["Table"] = "table";
        LuaLibraries["IO"] = "io";
        LuaLibraries["OS"] = "os";
        LuaLibraries["String"] = "string";
        LuaLibraries["UTF8"] = "utf8";
        LuaLibraries["Math"] = "math";
        LuaLibraries["Debug"] = "debug";
        LuaLibraries["Package"] = "package";
    })(exports.LuaLibraries || (exports.LuaLibraries = {}));
    class LuaTimeoutError extends Error {
    }

    class Decoration {
        constructor(target, options) {
            this.target = target;
            this.options = options;
        }
    }
    function decorate(target, options) {
        return new Decoration(target, options);
    }

    class Pointer extends Number {
    }

    class MultiReturn extends Array {
    }

    const INSTRUCTION_HOOK_COUNT = 1000;
    class Thread {
        constructor(lua, typeExtensions, address, parent) {
            this.closed = false;
            this.lua = lua;
            this.typeExtensions = typeExtensions;
            this.address = address;
            this.parent = parent;
        }
        newThread() {
            const address = this.lua.lua_newthread(this.address);
            if (!address) {
                throw new Error('lua_newthread returned a null pointer');
            }
            return new Thread(this.lua, this.typeExtensions, address);
        }
        resetThread() {
            this.assertOk(this.lua.lua_resetthread(this.address));
        }
        loadString(luaCode) {
            this.assertOk(this.lua.luaL_loadstring(this.address, luaCode));
        }
        loadFile(filename) {
            this.assertOk(this.lua.luaL_loadfilex(this.address, filename, null));
        }
        resume(argCount = 0) {
            const dataPointer = this.lua.module._malloc(PointerSize);
            try {
                this.lua.module.setValue(dataPointer, 0, 'i32');
                const luaResult = this.lua.lua_resume(this.address, null, argCount, dataPointer);
                return {
                    result: luaResult,
                    resultCount: this.lua.module.getValue(dataPointer, 'i32'),
                };
            }
            finally {
                this.lua.module._free(dataPointer);
            }
        }
        getTop() {
            return this.lua.lua_gettop(this.address);
        }
        setTop(index) {
            this.lua.lua_settop(this.address, index);
        }
        remove(index) {
            return this.lua.lua_remove(this.address, index);
        }
        setField(index, name, value) {
            index = this.lua.lua_absindex(this.address, index);
            this.pushValue(value);
            this.lua.lua_setfield(this.address, index, name);
        }
        async run(argCount = 0, options) {
            const originalTimeout = this.timeout;
            try {
                if ((options === null || options === void 0 ? void 0 : options.timeout) !== undefined) {
                    this.setTimeout(Date.now() + options.timeout);
                }
                let resumeResult = this.resume(argCount);
                while (resumeResult.result === exports.LuaReturn.Yield) {
                    if (this.timeout && Date.now() > this.timeout) {
                        if (resumeResult.resultCount > 0) {
                            this.pop(resumeResult.resultCount);
                        }
                        throw new LuaTimeoutError(`thread timeout exceeded`);
                    }
                    if (resumeResult.resultCount > 0) {
                        const lastValue = this.getValue(-1);
                        this.pop(resumeResult.resultCount);
                        if (lastValue === Promise.resolve(lastValue)) {
                            await lastValue;
                        }
                        else {
                            await new Promise((resolve) => setImmediate(resolve));
                        }
                    }
                    else {
                        await new Promise((resolve) => setImmediate(resolve));
                    }
                    resumeResult = this.resume(0);
                }
                this.assertOk(resumeResult.result);
                return this.getStackValues();
            }
            finally {
                if ((options === null || options === void 0 ? void 0 : options.timeout) !== undefined) {
                    this.setTimeout(originalTimeout);
                }
            }
        }
        runSync(argCount = 0) {
            this.assertOk(this.lua.lua_pcallk(this.address, argCount, LUA_MULTRET, 0, 0, null));
            return this.getStackValues();
        }
        pop(count = 1) {
            this.lua.lua_pop(this.address, count);
        }
        call(name, ...args) {
            const type = this.lua.lua_getglobal(this.address, name);
            if (type !== exports.LuaType.Function) {
                throw new Error(`A function of type '${type}' was pushed, expected is ${exports.LuaType.Function}`);
            }
            for (const arg of args) {
                this.pushValue(arg);
            }
            this.lua.lua_callk(this.address, args.length, LUA_MULTRET, 0, null);
            return this.getStackValues();
        }
        getStackValues() {
            const returns = this.getTop();
            const returnValues = new MultiReturn(returns);
            for (let i = 0; i < returns; i++) {
                returnValues[i] = this.getValue(i + 1);
            }
            return returnValues;
        }
        stateToThread(L) {
            var _a;
            return L === ((_a = this.parent) === null || _a === void 0 ? void 0 : _a.address) ? this.parent : new Thread(this.lua, this.typeExtensions, L, this.parent || this);
        }
        pushValue(rawValue, userdata) {
            var _a;
            const decoratedValue = this.getValueDecorations(rawValue);
            const target = (_a = decoratedValue.target) !== null && _a !== void 0 ? _a : undefined;
            if (target instanceof Thread) {
                const isMain = this.lua.lua_pushthread(target.address) === 1;
                if (!isMain) {
                    this.lua.lua_xmove(target.address, this.address, 1);
                }
                return;
            }
            const startTop = this.getTop();
            switch (typeof target) {
                case 'undefined':
                    this.lua.lua_pushnil(this.address);
                    break;
                case 'number':
                    if (Number.isInteger(target)) {
                        this.lua.lua_pushinteger(this.address, target);
                    }
                    else {
                        this.lua.lua_pushnumber(this.address, target);
                    }
                    break;
                case 'string':
                    this.lua.lua_pushstring(this.address, target);
                    break;
                case 'boolean':
                    this.lua.lua_pushboolean(this.address, target ? 1 : 0);
                    break;
                default:
                    if (!this.typeExtensions.find((wrapper) => wrapper.extension.pushValue(this, decoratedValue, userdata))) {
                        throw new Error(`The type '${typeof target}' is not supported by Lua`);
                    }
            }
            if (decoratedValue.options.metatable) {
                this.setMetatable(decoratedValue.options.metatable, -1);
            }
            if (this.getTop() !== startTop + 1) {
                throw new Error(`pushValue expected stack size ${startTop + 1}, got ${this.getTop()}`);
            }
        }
        setMetatable(metatable, index) {
            index = this.lua.lua_absindex(this.address, index);
            if (this.lua.lua_getmetatable(this.address, index)) {
                this.pop(1);
                const name = this.getMetatableName(index);
                throw new Error(`data already has associated metatable: ${name || 'unknown name'}`);
            }
            this.pushValue(metatable);
            this.lua.lua_setmetatable(this.address, index);
        }
        getMetatableName(index) {
            const metatableNameType = this.lua.luaL_getmetafield(this.address, index, '__name');
            if (metatableNameType === exports.LuaType.Nil) {
                return undefined;
            }
            if (metatableNameType !== exports.LuaType.String) {
                this.pop(1);
                return undefined;
            }
            const name = this.lua.lua_tolstring(this.address, -1, null);
            this.pop(1);
            return name;
        }
        getValue(idx, inputType, userdata) {
            idx = this.lua.lua_absindex(this.address, idx);
            const type = inputType !== null && inputType !== void 0 ? inputType : this.lua.lua_type(this.address, idx);
            switch (type) {
                case exports.LuaType.None:
                    return undefined;
                case exports.LuaType.Nil:
                    return null;
                case exports.LuaType.Number:
                    return this.lua.lua_tonumberx(this.address, idx, null);
                case exports.LuaType.String:
                    return this.lua.lua_tolstring(this.address, idx, null);
                case exports.LuaType.Boolean:
                    return Boolean(this.lua.lua_toboolean(this.address, idx));
                case exports.LuaType.Thread:
                    return this.stateToThread(this.lua.lua_tothread(this.address, idx));
                default: {
                    let metatableName;
                    if (type === exports.LuaType.Table || type === exports.LuaType.Userdata) {
                        metatableName = this.getMetatableName(idx);
                    }
                    const typeExtensionWrapper = this.typeExtensions.find((wrapper) => wrapper.extension.isType(this, idx, type, metatableName));
                    if (typeExtensionWrapper) {
                        return typeExtensionWrapper.extension.getValue(this, idx, userdata);
                    }
                    console.warn(`The type '${this.lua.lua_typename(this.address, type)}' returned is not supported on JS`);
                    return new Pointer(this.lua.lua_topointer(this.address, idx));
                }
            }
        }
        close() {
            if (this.isClosed()) {
                return;
            }
            if (this.hookFunctionPointer) {
                this.lua.module.removeFunction(this.hookFunctionPointer);
            }
            this.closed = true;
        }
        setTimeout(timeout) {
            if (timeout && timeout > 0) {
                if (!this.hookFunctionPointer) {
                    this.hookFunctionPointer = this.lua.module.addFunction(() => {
                        if (Date.now() > timeout) {
                            this.pushValue(new LuaTimeoutError(`thread timeout exceeded`));
                            this.lua.lua_error(this.address);
                        }
                    }, 'vii');
                }
                this.lua.lua_sethook(this.address, this.hookFunctionPointer, exports.LuaEventMasks.Count, INSTRUCTION_HOOK_COUNT);
                this.timeout = timeout;
            }
            else {
                this.timeout = undefined;
                this.lua.lua_sethook(this.address, null, 0, 0);
            }
        }
        getTimeout() {
            return this.timeout;
        }
        getPointer(index) {
            return new Pointer(this.lua.lua_topointer(this.address, index));
        }
        isClosed() {
            var _a;
            return !this.address || this.closed || Boolean((_a = this.parent) === null || _a === void 0 ? void 0 : _a.isClosed());
        }
        indexToString(index) {
            const str = this.lua.luaL_tolstring(this.address, index, null);
            this.pop();
            return str;
        }
        dumpStack(log = console.log) {
            const top = this.getTop();
            for (let i = 1; i <= top; i++) {
                const type = this.lua.lua_type(this.address, i);
                const typename = this.lua.lua_typename(this.address, type);
                const pointer = this.getPointer(i);
                const name = this.indexToString(i);
                const value = this.getValue(i, type);
                log(i, typename, pointer, name, value);
            }
        }
        assertOk(result) {
            if (result !== exports.LuaReturn.Ok && result !== exports.LuaReturn.Yield) {
                const resultString = exports.LuaReturn[result];
                const error = new Error(`Lua Error(${resultString}/${result})`);
                if (this.getTop() > 0) {
                    if (result === exports.LuaReturn.ErrorMem) {
                        error.message = this.lua.lua_tolstring(this.address, -1, null);
                    }
                    else {
                        const luaError = this.getValue(-1);
                        if (luaError instanceof Error) {
                            error.stack = luaError.stack;
                        }
                        error.message = this.indexToString(-1);
                    }
                }
                throw error;
            }
        }
        getValueDecorations(value) {
            return value instanceof Decoration ? value : new Decoration(value, {});
        }
    }

    class Global extends Thread {
        constructor(cmodule, shouldTraceAllocations) {
            if (shouldTraceAllocations) {
                const memoryStats = { memoryUsed: 0 };
                const allocatorFunctionPointer = cmodule.module.addFunction((_userData, pointer, oldSize, newSize) => {
                    if (newSize === 0 && pointer) {
                        memoryStats.memoryUsed -= oldSize;
                        cmodule.module._free(pointer);
                        return null;
                    }
                    const endMemoryDelta = pointer ? newSize - oldSize : newSize;
                    const endMemory = memoryStats.memoryUsed + endMemoryDelta;
                    if (newSize > oldSize && memoryStats.memoryMax && endMemory > memoryStats.memoryMax) {
                        return null;
                    }
                    const reallocated = cmodule.module._realloc(pointer, newSize);
                    if (reallocated) {
                        memoryStats.memoryUsed = endMemory;
                    }
                    return reallocated;
                }, 'iiiii');
                super(cmodule, [], cmodule.lua_newstate(allocatorFunctionPointer, null));
                this.memoryStats = memoryStats;
                this.allocatorFunctionPointer = allocatorFunctionPointer;
            }
            else {
                super(cmodule, [], cmodule.luaL_newstate());
            }
            if (this.isClosed()) {
                throw new Error('Global state could not be created (probably due to lack of memory)');
            }
        }
        close() {
            if (this.isClosed()) {
                return;
            }
            super.close();
            this.lua.lua_close(this.address);
            if (this.allocatorFunctionPointer) {
                this.lua.module.removeFunction(this.allocatorFunctionPointer);
            }
            for (const wrapper of this.typeExtensions) {
                wrapper.extension.close();
            }
        }
        registerTypeExtension(priority, extension) {
            this.typeExtensions.push({ extension, priority });
            this.typeExtensions.sort((a, b) => b.priority - a.priority);
        }
        loadLibrary(library) {
            switch (library) {
                case exports.LuaLibraries.Base:
                    this.lua.luaopen_base(this.address);
                    break;
                case exports.LuaLibraries.Coroutine:
                    this.lua.luaopen_coroutine(this.address);
                    break;
                case exports.LuaLibraries.Table:
                    this.lua.luaopen_table(this.address);
                    break;
                case exports.LuaLibraries.IO:
                    this.lua.luaopen_io(this.address);
                    break;
                case exports.LuaLibraries.OS:
                    this.lua.luaopen_os(this.address);
                    break;
                case exports.LuaLibraries.String:
                    this.lua.luaopen_string(this.address);
                    break;
                case exports.LuaLibraries.UTF8:
                    this.lua.luaopen_string(this.address);
                    break;
                case exports.LuaLibraries.Math:
                    this.lua.luaopen_math(this.address);
                    break;
                case exports.LuaLibraries.Debug:
                    this.lua.luaopen_debug(this.address);
                    break;
                case exports.LuaLibraries.Package:
                    this.lua.luaopen_package(this.address);
                    break;
            }
            this.lua.lua_setglobal(this.address, library);
        }
        get(name) {
            const type = this.lua.lua_getglobal(this.address, name);
            const value = this.getValue(-1, type);
            this.pop();
            return value;
        }
        set(name, value) {
            this.pushValue(value);
            this.lua.lua_setglobal(this.address, name);
        }
        getTable(name, callback) {
            const startStackTop = this.getTop();
            const type = this.lua.lua_getglobal(this.address, name);
            try {
                if (type !== exports.LuaType.Table) {
                    throw new TypeError(`Unexpected type in ${name}. Expected ${exports.LuaType[exports.LuaType.Table]}. Got ${exports.LuaType[type]}.`);
                }
                callback(startStackTop + 1);
            }
            finally {
                if (this.getTop() !== startStackTop + 1) {
                    console.warn(`getTable: expected stack size ${startStackTop} got ${this.getTop()}`);
                }
                this.setTop(startStackTop);
            }
        }
        getMemoryUsed() {
            return this.getMemoryStatsRef().memoryUsed;
        }
        getMemoryMax() {
            return this.getMemoryStatsRef().memoryMax;
        }
        setMemoryMax(max) {
            this.getMemoryStatsRef().memoryMax = max;
        }
        getMemoryStatsRef() {
            if (!this.memoryStats) {
                throw new Error('Memory allocations is not being traced, please build engine with { traceAllocations: true }');
            }
            return this.memoryStats;
        }
    }

    class LuaTypeExtension {
        constructor(thread, name) {
            this.thread = thread;
            this.name = name;
        }
        isType(_thread, _index, type, name) {
            return type === exports.LuaType.Userdata && name === this.name;
        }
        getValue(thread, index, _userdata) {
            const refUserdata = thread.lua.luaL_testudata(thread.address, index, this.name);
            if (!refUserdata) {
                throw new Error(`data does not have the expected metatable: ${this.name}`);
            }
            const referencePointer = thread.lua.module.getValue(refUserdata, '*');
            return thread.lua.getRef(referencePointer);
        }
        pushValue(thread, decoratedValue, _userdata) {
            const { target } = decoratedValue;
            const pointer = thread.lua.ref(target);
            const userDataPointer = thread.lua.lua_newuserdatauv(thread.address, PointerSize, 0);
            thread.lua.module.setValue(userDataPointer, pointer, '*');
            if (exports.LuaType.Nil === thread.lua.luaL_getmetatable(thread.address, this.name)) {
                thread.pop(2);
                throw new Error(`metatable not found: ${this.name}`);
            }
            thread.lua.lua_setmetatable(thread.address, -2);
            return true;
        }
    }

    class ErrorTypeExtension extends LuaTypeExtension {
        constructor(thread, injectObject) {
            super(thread, 'js_error');
            this.gcPointer = thread.lua.module.addFunction((functionStateAddress) => {
                const userDataPointer = thread.lua.luaL_checkudata(functionStateAddress, 1, this.name);
                const referencePointer = thread.lua.module.getValue(userDataPointer, '*');
                thread.lua.unref(referencePointer);
                return exports.LuaReturn.Ok;
            }, 'ii');
            if (thread.lua.luaL_newmetatable(thread.address, this.name)) {
                const metatableIndex = thread.lua.lua_gettop(thread.address);
                thread.lua.lua_pushstring(thread.address, 'protected metatable');
                thread.lua.lua_setfield(thread.address, metatableIndex, '__metatable');
                thread.lua.lua_pushcclosure(thread.address, this.gcPointer, 0);
                thread.lua.lua_setfield(thread.address, metatableIndex, '__gc');
                thread.pushValue((jsRefError, key) => {
                    if (key === 'message') {
                        return jsRefError.message;
                    }
                    return null;
                });
                thread.lua.lua_setfield(thread.address, metatableIndex, '__index');
                thread.pushValue((jsRefError) => {
                    return jsRefError.message;
                });
                thread.lua.lua_setfield(thread.address, metatableIndex, '__tostring');
            }
            thread.lua.lua_pop(thread.address, 1);
            if (injectObject) {
                thread.set('Error', {
                    create: (message) => {
                        if (message && typeof message !== 'string') {
                            throw new Error('message must be a string');
                        }
                        return new Error(message);
                    },
                });
            }
        }
        pushValue(thread, decoration) {
            if (!(decoration.target instanceof Error)) {
                return false;
            }
            return super.pushValue(thread, decoration);
        }
        close() {
            this.thread.lua.module.removeFunction(this.gcPointer);
        }
    }
    function createTypeExtension$5(thread, injectObject) {
        return new ErrorTypeExtension(thread, injectObject);
    }

    class RawResult {
        constructor(count) {
            this.count = count;
        }
    }

    function decorateFunction(target, options) {
        return new Decoration(target, options);
    }
    class FunctionTypeExtension extends LuaTypeExtension {
        constructor(thread) {
            super(thread, 'js_function');
            this.functionRegistry = typeof FinalizationRegistry !== 'undefined'
                ? new FinalizationRegistry((func) => {
                    if (!this.thread.isClosed()) {
                        this.thread.lua.luaL_unref(this.thread.address, LUA_REGISTRYINDEX, func);
                    }
                })
                : undefined;
            if (!this.functionRegistry) {
                console.warn('FunctionTypeExtension: FinalizationRegistry not found. Memory leaks likely.');
            }
            this.gcPointer = thread.lua.module.addFunction((calledL) => {
                const userDataPointer = thread.lua.luaL_checkudata(calledL, 1, this.name);
                const functionPointer = thread.lua.module.getValue(userDataPointer, '*');
                thread.lua.module.removeFunction(functionPointer);
                return exports.LuaReturn.Ok;
            }, 'ii');
            if (thread.lua.luaL_newmetatable(thread.address, this.name)) {
                thread.lua.lua_pushstring(thread.address, '__gc');
                thread.lua.lua_pushcclosure(thread.address, this.gcPointer, 0);
                thread.lua.lua_settable(thread.address, -3);
                thread.lua.lua_pushstring(thread.address, '__metatable');
                thread.lua.lua_pushstring(thread.address, 'protected metatable');
                thread.lua.lua_settable(thread.address, -3);
            }
            thread.lua.lua_pop(thread.address, 1);
        }
        close() {
            this.thread.lua.module.removeFunction(this.gcPointer);
        }
        isType(_thread, _index, type) {
            return type === exports.LuaType.Function;
        }
        pushValue(thread, { target, options }) {
            if (typeof target !== 'function') {
                return false;
            }
            const pointer = thread.lua.module.addFunction((calledL) => {
                const calledThread = thread.stateToThread(calledL);
                const argsQuantity = calledThread.getTop();
                const args = [];
                if (options.receiveThread) {
                    args.push(calledThread);
                }
                if (options.receiveArgsQuantity) {
                    args.push(argsQuantity);
                }
                else {
                    for (let i = 1; i <= argsQuantity; i++) {
                        const value = calledThread.getValue(i);
                        if (i !== 1 || !(options === null || options === void 0 ? void 0 : options.self) || value !== options.self) {
                            args.push(value);
                        }
                    }
                }
                try {
                    const result = target.apply(options === null || options === void 0 ? void 0 : options.self, args);
                    if (result === undefined) {
                        return 0;
                    }
                    else if (result instanceof RawResult) {
                        return result.count;
                    }
                    else if (result instanceof MultiReturn) {
                        for (const item of result) {
                            calledThread.pushValue(item);
                        }
                        return result.length;
                    }
                    else {
                        calledThread.pushValue(result);
                        return 1;
                    }
                }
                catch (err) {
                    if (err === 'longjmp') {
                        throw err;
                    }
                    calledThread.pushValue(err);
                    return calledThread.lua.lua_error(calledThread.address);
                }
            }, 'ii');
            this.createAndPushFunctionReference(thread, pointer);
            thread.lua.lua_pushcclosure(thread.address, pointer, 1);
            return true;
        }
        getValue(thread, index) {
            var _a;
            thread.lua.lua_pushvalue(thread.address, index);
            const func = thread.lua.luaL_ref(thread.address, LUA_REGISTRYINDEX);
            const jsFunc = (...args) => {
                if (thread.isClosed()) {
                    console.warn('Tried to call a function after closing lua state');
                    return;
                }
                const internalType = thread.lua.lua_rawgeti(thread.address, LUA_REGISTRYINDEX, func);
                if (internalType !== exports.LuaType.Function) {
                    throw new Error(`A function of type '${internalType}' was pushed, expected is ${exports.LuaType.Function}`);
                }
                for (const arg of args) {
                    thread.pushValue(arg);
                }
                const status = thread.lua.lua_pcallk(thread.address, args.length, 1, 0, 0, null);
                if (status === exports.LuaReturn.Yield) {
                    throw new Error('cannot yield in callbacks from javascript');
                }
                thread.assertOk(status);
                const result = thread.getValue(-1);
                thread.pop();
                return result;
            };
            (_a = this.functionRegistry) === null || _a === void 0 ? void 0 : _a.register(jsFunc, func);
            return jsFunc;
        }
        createAndPushFunctionReference(thread, pointer) {
            const userDataPointer = thread.lua.lua_newuserdatauv(thread.address, PointerSize, 0);
            thread.lua.module.setValue(userDataPointer, pointer, '*');
            if (exports.LuaType.Nil === thread.lua.luaL_getmetatable(thread.address, this.name)) {
                thread.pop(2);
                throw new Error(`metatable not found: ${this.name}`);
            }
            thread.lua.lua_setmetatable(thread.address, -2);
        }
    }
    function createTypeExtension$4(thread) {
        return new FunctionTypeExtension(thread);
    }

    class PromiseTypeExtension extends LuaTypeExtension {
        constructor(thread, injectObject) {
            super(thread, 'js_promise');
            this.functionRegistry = typeof FinalizationRegistry !== 'undefined'
                ? new FinalizationRegistry((func) => {
                    this.thread.lua.module.removeFunction(func);
                })
                : undefined;
            if (!this.functionRegistry) {
                console.warn('PromiseTypeExtension: FinalizationRegistry not found. Memory leaks likely.');
            }
            this.gcPointer = thread.lua.module.addFunction((functionStateAddress) => {
                const userDataPointer = thread.lua.luaL_checkudata(functionStateAddress, 1, this.name);
                const referencePointer = thread.lua.module.getValue(userDataPointer, '*');
                thread.lua.unref(referencePointer);
                return exports.LuaReturn.Ok;
            }, 'ii');
            if (thread.lua.luaL_newmetatable(thread.address, this.name)) {
                const metatableIndex = thread.lua.lua_gettop(thread.address);
                thread.lua.lua_pushstring(thread.address, 'protected metatable');
                thread.lua.lua_setfield(thread.address, metatableIndex, '__metatable');
                thread.lua.lua_pushcclosure(thread.address, this.gcPointer, 0);
                thread.lua.lua_setfield(thread.address, metatableIndex, '__gc');
                const checkSelf = (self) => {
                    if (Promise.resolve(self) !== self) {
                        throw new Error('promise method called without self instance');
                    }
                    return true;
                };
                thread.pushValue({
                    next: (self, ...args) => checkSelf(self) && self.then(...args),
                    catch: (self, ...args) => checkSelf(self) && self.catch(...args),
                    finally: (self, ...args) => checkSelf(self) && self.finally(...args),
                    await: decorateFunction((functionThread, self) => {
                        var _a;
                        checkSelf(self);
                        let promiseResult = undefined;
                        const awaitPromise = self
                            .then((res) => {
                            promiseResult = { status: 'fulfilled', value: res };
                        })
                            .catch((err) => {
                            promiseResult = { status: 'rejected', value: err };
                        });
                        const continuance = this.thread.lua.module.addFunction((continuanceState) => {
                            if (!promiseResult) {
                                return thread.lua.lua_yieldk(functionThread.address, 0, 0, continuance);
                            }
                            const continuanceThread = thread.stateToThread(continuanceState);
                            if (promiseResult.status === 'rejected') {
                                continuanceThread.pushValue(promiseResult.value || new Error('promise rejected with no error'));
                                return this.thread.lua.lua_error(continuanceState);
                            }
                            if (promiseResult.value instanceof RawResult) {
                                return promiseResult.value.count;
                            }
                            else if (promiseResult.value instanceof MultiReturn) {
                                for (const arg of promiseResult.value) {
                                    continuanceThread.pushValue(arg);
                                }
                                return promiseResult.value.length;
                            }
                            else {
                                continuanceThread.pushValue(promiseResult.value);
                                return 1;
                            }
                        }, 'iiii');
                        (_a = this.functionRegistry) === null || _a === void 0 ? void 0 : _a.register(awaitPromise, continuance);
                        functionThread.pushValue(awaitPromise);
                        return new RawResult(thread.lua.lua_yieldk(functionThread.address, 1, 0, continuance));
                    }, { receiveThread: true }),
                });
                thread.lua.lua_setfield(thread.address, metatableIndex, '__index');
                thread.pushValue((self, other) => self === other);
                thread.lua.lua_setfield(thread.address, metatableIndex, '__eq');
            }
            thread.lua.lua_pop(thread.address, 1);
            if (injectObject) {
                thread.set('Promise', {
                    create: (callback) => new Promise(callback),
                    all: (promiseArray) => {
                        if (!Array.isArray(promiseArray)) {
                            throw new Error('argument must be an array of promises');
                        }
                        return Promise.all(promiseArray.map((potentialPromise) => Promise.resolve(potentialPromise)));
                    },
                    resolve: (value) => Promise.resolve(value),
                });
            }
        }
        close() {
            this.thread.lua.module.removeFunction(this.gcPointer);
        }
        pushValue(thread, decoration) {
            if (Promise.resolve(decoration.target) !== decoration.target) {
                return false;
            }
            return super.pushValue(thread, decoration);
        }
    }
    function createTypeExtension$3(thread, injectObject) {
        return new PromiseTypeExtension(thread, injectObject);
    }

    function decorateProxy(target, options) {
        return new Decoration(target, options || {});
    }
    class ProxyTypeExtension extends LuaTypeExtension {
        constructor(thread) {
            super(thread, 'js_proxy');
            this.gcPointer = thread.lua.module.addFunction((functionStateAddress) => {
                const userDataPointer = thread.lua.luaL_checkudata(functionStateAddress, 1, this.name);
                const referencePointer = thread.lua.module.getValue(userDataPointer, '*');
                thread.lua.unref(referencePointer);
                return exports.LuaReturn.Ok;
            }, 'ii');
            if (thread.lua.luaL_newmetatable(thread.address, this.name)) {
                const metatableIndex = thread.lua.lua_gettop(thread.address);
                thread.lua.lua_pushstring(thread.address, 'protected metatable');
                thread.lua.lua_setfield(thread.address, metatableIndex, '__metatable');
                thread.lua.lua_pushcclosure(thread.address, this.gcPointer, 0);
                thread.lua.lua_setfield(thread.address, metatableIndex, '__gc');
                thread.pushValue((self, key) => {
                    switch (typeof key) {
                        case 'number':
                            key = key - 1;
                        case 'string':
                            break;
                        default:
                            throw new Error('Only strings or numbers can index js objects');
                    }
                    const value = self[key];
                    if (typeof value === 'function') {
                        return decorateFunction(value, { self });
                    }
                    return value;
                });
                thread.lua.lua_setfield(thread.address, metatableIndex, '__index');
                thread.pushValue((self, key, value) => {
                    switch (typeof key) {
                        case 'number':
                            key = key - 1;
                        case 'string':
                            break;
                        default:
                            throw new Error('Only strings or numbers can index js objects');
                    }
                    self[key] = value;
                });
                thread.lua.lua_setfield(thread.address, metatableIndex, '__newindex');
                thread.pushValue((self) => {
                    var _a, _b;
                    return (_b = (_a = self.toString) === null || _a === void 0 ? void 0 : _a.call(self)) !== null && _b !== void 0 ? _b : typeof self;
                });
                thread.lua.lua_setfield(thread.address, metatableIndex, '__tostring');
                thread.pushValue((self) => {
                    return self.length || 0;
                });
                thread.lua.lua_setfield(thread.address, metatableIndex, '__len');
                thread.pushValue((self) => {
                    const keys = Object.getOwnPropertyNames(self);
                    let i = 0;
                    return MultiReturn.of(() => {
                        const ret = MultiReturn.of(keys[i], self[keys[i]]);
                        i++;
                        return ret;
                    }, self, null);
                });
                thread.lua.lua_setfield(thread.address, metatableIndex, '__pairs');
                thread.pushValue((self, other) => {
                    return self === other;
                });
                thread.lua.lua_setfield(thread.address, metatableIndex, '__eq');
                thread.pushValue((self, ...args) => {
                    if (args[0] === self) {
                        args.shift();
                    }
                    return self(...args);
                });
                thread.lua.lua_setfield(thread.address, metatableIndex, '__call');
            }
            thread.lua.lua_pop(thread.address, 1);
        }
        isType(_thread, _index, type, name) {
            return type === exports.LuaType.Userdata && name === this.name;
        }
        getValue(thread, index) {
            const refUserdata = thread.lua.lua_touserdata(thread.address, index);
            const referencePointer = thread.lua.module.getValue(refUserdata, '*');
            return thread.lua.getRef(referencePointer);
        }
        pushValue(thread, decoratedValue) {
            var _a;
            const { target, options } = decoratedValue;
            if (options.proxy === undefined) {
                if (target === null || target === undefined) {
                    return false;
                }
                if (typeof target !== 'object') {
                    const isClass = typeof target === 'function' && ((_a = target.prototype) === null || _a === void 0 ? void 0 : _a.constructor) === target && target.toString().startsWith('class ');
                    if (!isClass) {
                        return false;
                    }
                }
                if (Promise.resolve(target) === target) {
                    return false;
                }
            }
            else if (options.proxy === false) {
                return false;
            }
            if (options.metatable && !(options.metatable instanceof Decoration)) {
                decoratedValue.options.metatable = decorateProxy(options.metatable, { proxy: false });
                return false;
            }
            return super.pushValue(thread, decoratedValue);
        }
        close() {
            this.thread.lua.module.removeFunction(this.gcPointer);
        }
    }
    function createTypeExtension$2(thread) {
        return new ProxyTypeExtension(thread);
    }

    class TableTypeExtension extends LuaTypeExtension {
        constructor(thread) {
            super(thread, 'js_table');
        }
        close() {
        }
        isType(_thread, _index, type) {
            return type === exports.LuaType.Table;
        }
        getValue(thread, index, userdata) {
            const seenMap = userdata || new Map();
            const pointer = thread.lua.lua_topointer(thread.address, index);
            let table = seenMap.get(pointer);
            if (!table) {
                table = {};
                seenMap.set(pointer, table);
                this.tableToObject(thread, index, seenMap, table);
            }
            const keys = Object.keys(table);
            const isSequential = keys.length > 0 && keys.every((key, index) => key === String(index + 1));
            return isSequential ? Object.values(table) : table;
        }
        pushValue(thread, { target }, userdata) {
            if (typeof target !== 'object' || target === null) {
                return false;
            }
            const seenMap = userdata || new Map();
            const existingReference = seenMap.get(target);
            if (existingReference !== undefined) {
                thread.lua.lua_rawgeti(thread.address, LUA_REGISTRYINDEX, existingReference);
                return true;
            }
            try {
                const tableIndex = thread.getTop() + 1;
                const createTable = (arrayCount, keyCount) => {
                    thread.lua.lua_createtable(thread.address, arrayCount, keyCount);
                    const ref = thread.lua.luaL_ref(thread.address, LUA_REGISTRYINDEX);
                    seenMap.set(target, ref);
                    thread.lua.lua_rawgeti(thread.address, LUA_REGISTRYINDEX, ref);
                };
                if (Array.isArray(target)) {
                    createTable(target.length, 0);
                    for (let i = 0; i < target.length; i++) {
                        thread.pushValue(i + 1, seenMap);
                        thread.pushValue(target[i], seenMap);
                        thread.lua.lua_settable(thread.address, tableIndex);
                    }
                }
                else {
                    createTable(0, Object.getOwnPropertyNames(target).length);
                    for (const key in target) {
                        thread.pushValue(key, seenMap);
                        thread.pushValue(target[key], seenMap);
                        thread.lua.lua_settable(thread.address, tableIndex);
                    }
                }
            }
            finally {
                if (userdata === undefined) {
                    for (const reference of seenMap.values()) {
                        thread.lua.luaL_unref(thread.address, LUA_REGISTRYINDEX, reference);
                    }
                }
            }
            return true;
        }
        tableToObject(thread, index, seenMap, table) {
            thread.lua.lua_pushnil(thread.address);
            while (thread.lua.lua_next(thread.address, index)) {
                const key = thread.lua.luaL_tolstring(thread.address, -2, null);
                thread.pop();
                const value = thread.getValue(-1, undefined, seenMap);
                table[key] = value;
                thread.pop();
            }
        }
    }
    function createTypeExtension$1(thread) {
        return new TableTypeExtension(thread);
    }

    function decorateUserdata(target) {
        return new Decoration(target, { reference: true });
    }
    class UserdataTypeExtension extends LuaTypeExtension {
        constructor(thread) {
            super(thread, 'js_userdata');
            this.gcPointer = thread.lua.module.addFunction((functionStateAddress) => {
                const userDataPointer = thread.lua.luaL_checkudata(functionStateAddress, 1, this.name);
                const referencePointer = thread.lua.module.getValue(userDataPointer, '*');
                thread.lua.unref(referencePointer);
                return exports.LuaReturn.Ok;
            }, 'ii');
            if (thread.lua.luaL_newmetatable(thread.address, this.name)) {
                const metatableIndex = thread.lua.lua_gettop(thread.address);
                thread.lua.lua_pushstring(thread.address, 'protected metatable');
                thread.lua.lua_setfield(thread.address, metatableIndex, '__metatable');
                thread.lua.lua_pushcclosure(thread.address, this.gcPointer, 0);
                thread.lua.lua_setfield(thread.address, metatableIndex, '__gc');
            }
            thread.lua.lua_pop(thread.address, 1);
        }
        isType(_thread, _index, type, name) {
            return type === exports.LuaType.Userdata && name === this.name;
        }
        getValue(thread, index) {
            const refUserdata = thread.lua.lua_touserdata(thread.address, index);
            const referencePointer = thread.lua.module.getValue(refUserdata, '*');
            return thread.lua.getRef(referencePointer);
        }
        pushValue(thread, decoratedValue) {
            if (!decoratedValue.options.reference) {
                return false;
            }
            return super.pushValue(thread, decoratedValue);
        }
        close() {
            this.thread.lua.module.removeFunction(this.gcPointer);
        }
    }
    function createTypeExtension(thread) {
        return new UserdataTypeExtension(thread);
    }

    class LuaEngine {
        constructor(cmodule, { openStandardLibs = true, injectObjects = false, enableProxy = true, traceAllocations = false } = {}) {
            this.cmodule = cmodule;
            this.global = new Global(this.cmodule, traceAllocations);
            this.global.registerTypeExtension(0, createTypeExtension$1(this.global));
            this.global.registerTypeExtension(0, createTypeExtension$4(this.global));
            this.global.registerTypeExtension(1, createTypeExtension$3(this.global, injectObjects));
            if (enableProxy) {
                this.global.registerTypeExtension(3, createTypeExtension$2(this.global));
            }
            else {
                this.global.registerTypeExtension(1, createTypeExtension$5(this.global, injectObjects));
            }
            this.global.registerTypeExtension(4, createTypeExtension(this.global));
            if (openStandardLibs) {
                this.cmodule.luaL_openlibs(this.global.address);
            }
        }
        doString(script) {
            return this.callByteCode((thread) => thread.loadString(script));
        }
        doFile(filename) {
            return this.callByteCode((thread) => thread.loadFile(filename));
        }
        doStringSync(script) {
            this.global.loadString(script);
            const result = this.global.runSync();
            return result[0];
        }
        doFileSync(filename) {
            this.global.loadFile(filename);
            const result = this.global.runSync();
            return result[0];
        }
        async callByteCode(loader) {
            const thread = this.global.newThread();
            const threadIndex = this.global.getTop();
            try {
                loader(thread);
                const result = await thread.run(0);
                if (result.length > 0) {
                    this.cmodule.lua_xmove(thread.address, this.global.address, result.length);
                }
                return result[0];
            }
            finally {
                this.global.remove(threadIndex);
            }
        }
    }

    var initWasmModule = (() => {
      var _scriptDir = (typeof document === 'undefined' && typeof location === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __filename).href : typeof document === 'undefined' ? location.href : (document.currentScript && document.currentScript.src || new URL('index.js', document.baseURI).href));
      
      return (
    function(initWasmModule) {
      initWasmModule = initWasmModule || {};
    var c;c||(c=typeof initWasmModule !== 'undefined' ? initWasmModule : {});var aa=Object.assign,ba,ca;c.ready=new Promise(function(a,b){ba=a;ca=b;});var da=aa({},c),ea="./this.program",fa=(a,b)=>{throw b;},ha="object"===typeof window,e="function"===typeof importScripts,m="object"===typeof process&&"object"===typeof process.versions&&"string"===typeof process.versions.node,u="",x,z,ia,fs,ja,ka;
    if(m)u=e?require("path").dirname(u)+"/":__dirname+"/",ka=function(){ja||(fs=require("fs"),ja=require("path"));},x=function(a,b){ka();a=ja.normalize(a);return fs.readFileSync(a,b?null:"utf8")},ia=function(a){a=x(a,!0);a.buffer||(a=new Uint8Array(a));return a},z=function(a,b,d){ka();a=ja.normalize(a);fs.readFile(a,function(f,g){f?d(f):b(g.buffer);});},1<process.argv.length&&(ea=process.argv[1].replace(/\\/g,"/")),process.argv.slice(2),fa=(a,b)=>{if(noExitRuntime||0<la)throw process.exitCode=a,b;b instanceof
    ma||A("exiting due to exception: "+b);process.exit(a);},c.inspect=function(){return "[Emscripten Module object]"};else if(ha||e)e?u=self.location.href:"undefined"!==typeof document&&document.currentScript&&(u=document.currentScript.src),_scriptDir&&(u=_scriptDir),0!==u.indexOf("blob:")?u=u.substr(0,u.replace(/[?#].*/,"").lastIndexOf("/")+1):u="",x=function(a){var b=new XMLHttpRequest;b.open("GET",a,!1);b.send(null);return b.responseText},e&&(ia=function(a){var b=new XMLHttpRequest;b.open("GET",a,!1);
    b.responseType="arraybuffer";b.send(null);return new Uint8Array(b.response)}),z=function(a,b,d){var f=new XMLHttpRequest;f.open("GET",a,!0);f.responseType="arraybuffer";f.onload=function(){200==f.status||0==f.status&&f.response?b(f.response):d();};f.onerror=d;f.send(null);};var na=c.print||console.log.bind(console),A=c.printErr||console.warn.bind(console);aa(c,da);da=null;c.thisProgram&&(ea=c.thisProgram);c.quit&&(fa=c.quit);var oa=[],D,pa=0,E;c.wasmBinary&&(E=c.wasmBinary);
    var noExitRuntime=c.noExitRuntime||!0;"object"!==typeof WebAssembly&&F("no native wasm support detected");var qa,ra=!1;
    function sa(a,b,d,f){var g={string:function(n){var q=0;if(null!==n&&void 0!==n&&0!==n){var w=(n.length<<2)+1;q=ta(w);ua(n,G,q,w);}return q},array:function(n){var q=ta(n.length);H.set(n,q);return q}};a=c["_"+a];var h=[],l=0;if(f)for(var p=0;p<f.length;p++){var t=g[d[p]];t?(0===l&&(l=va()),h[p]=t(f[p])):h[p]=f[p];}d=a.apply(null,h);return d=function(n){0!==l&&wa(l);return "string"===b?J(n):"boolean"===b?!!n:n}(d)}var xa="undefined"!==typeof TextDecoder?new TextDecoder("utf8"):void 0;
    function K(a,b,d){var f=b+d;for(d=b;a[d]&&!(d>=f);)++d;if(16<d-b&&a.subarray&&xa)return xa.decode(a.subarray(b,d));for(f="";b<d;){var g=a[b++];if(g&128){var h=a[b++]&63;if(192==(g&224))f+=String.fromCharCode((g&31)<<6|h);else {var l=a[b++]&63;g=224==(g&240)?(g&15)<<12|h<<6|l:(g&7)<<18|h<<12|l<<6|a[b++]&63;65536>g?f+=String.fromCharCode(g):(g-=65536,f+=String.fromCharCode(55296|g>>10,56320|g&1023));}}else f+=String.fromCharCode(g);}return f}function J(a){return a?K(G,a,void 0):""}
    function ua(a,b,d,f){if(!(0<f))return 0;var g=d;f=d+f-1;for(var h=0;h<a.length;++h){var l=a.charCodeAt(h);if(55296<=l&&57343>=l){var p=a.charCodeAt(++h);l=65536+((l&1023)<<10)|p&1023;}if(127>=l){if(d>=f)break;b[d++]=l;}else {if(2047>=l){if(d+1>=f)break;b[d++]=192|l>>6;}else {if(65535>=l){if(d+2>=f)break;b[d++]=224|l>>12;}else {if(d+3>=f)break;b[d++]=240|l>>18;b[d++]=128|l>>12&63;}b[d++]=128|l>>6&63;}b[d++]=128|l&63;}}b[d]=0;return d-g}
    function ya(a){for(var b=0,d=0;d<a.length;++d){var f=a.charCodeAt(d);55296<=f&&57343>=f&&(f=65536+((f&1023)<<10)|a.charCodeAt(++d)&1023);127>=f?++b:b=2047>=f?b+2:65535>=f?b+3:b+4;}return b}function za(a){var b=ya(a)+1,d=Aa(b);d&&ua(a,H,d,b);return d}var Ba,H,G,Ca,L,Da,Ea;
    function Fa(){var a=qa.buffer;Ba=a;c.HEAP8=H=new Int8Array(a);c.HEAP16=Ca=new Int16Array(a);c.HEAP32=L=new Int32Array(a);c.HEAPU8=G=new Uint8Array(a);c.HEAPU16=new Uint16Array(a);c.HEAPU32=new Uint32Array(a);c.HEAPF32=Da=new Float32Array(a);c.HEAPF64=Ea=new Float64Array(a);}var M,Ga=[],Ha=[],Ia=[],la=0;function Ja(){var a=c.preRun.shift();Ga.unshift(a);}var N=0,La=null;function Ma(){N++;c.monitorRunDependencies&&c.monitorRunDependencies(N);}
    function Na(){N--;c.monitorRunDependencies&&c.monitorRunDependencies(N);if(0==N&&(La)){var a=La;La=null;a();}}c.preloadedImages={};c.preloadedAudios={};function F(a){if(c.onAbort)c.onAbort(a);a="Aborted("+a+")";A(a);ra=!0;a=new WebAssembly.RuntimeError(a+". Build with -s ASSERTIONS=1 for more info.");ca(a);throw a;}function Oa(){return O.startsWith("data:application/octet-stream;base64,")}var O;
    if(c.locateFile){if(O="glue.wasm",!Oa()){var Pa=O;O=c.locateFile?c.locateFile(Pa,u):u+Pa;}}else O=(new URL("glue.wasm",(typeof document === 'undefined' && typeof location === 'undefined' ? new (require('u' + 'rl').URL)('file:' + __filename).href : typeof document === 'undefined' ? location.href : (document.currentScript && document.currentScript.src || new URL('index.js', document.baseURI).href)))).toString();function Qa(){var a=O;try{if(a==O&&E)return new Uint8Array(E);if(ia)return ia(a);throw "both async and sync fetching of the wasm failed";}catch(b){F(b);}}
    function Ra(){if(!E&&(ha||e)){if("function"===typeof fetch&&!O.startsWith("file://"))return fetch(O,{credentials:"same-origin"}).then(function(a){if(!a.ok)throw "failed to load wasm binary file at '"+O+"'";return a.arrayBuffer()}).catch(function(){return Qa()});if(z)return new Promise(function(a,b){z(O,function(d){a(new Uint8Array(d));},b);})}return Promise.resolve().then(function(){return Qa()})}var P,Q;
    function Sa(a){for(;0<a.length;){var b=a.shift();if("function"==typeof b)b(c);else {var d=b.bf;"number"===typeof d?void 0===b.Cd?Ta(d)():Ta(d)(b.Cd):d(void 0===b.Cd?null:b.Cd);}}}var R=[];function Ta(a){var b=R[a];b||(a>=R.length&&(R.length=a+1),R[a]=b=M.get(a));return b}var Ua;Ua=m?()=>{var a=process.hrtime();return 1E3*a[0]+a[1]/1E6}:()=>performance.now();var Va;
    function Wa(){function a(l){return (l=l.toTimeString().match(/\(([A-Za-z ]+)\)$/))?l[1]:"GMT"}var b=(new Date).getFullYear(),d=new Date(b,0,1),f=new Date(b,6,1);b=d.getTimezoneOffset();var g=f.getTimezoneOffset(),h=Math.max(b,g);L[Xa()>>2]=60*h;L[Ya()>>2]=Number(b!=g);d=a(d);f=a(f);d=za(d);f=za(f);g<b?(L[S()>>2]=d,L[S()+4>>2]=f):(L[S()>>2]=f,L[S()+4>>2]=d);}var Za;
    function $a(a,b){for(var d=0,f=a.length-1;0<=f;f--){var g=a[f];"."===g?a.splice(f,1):".."===g?(a.splice(f,1),d++):d&&(a.splice(f,1),d--);}if(b)for(;d;d--)a.unshift("..");return a}function ab(a){var b="/"===a.charAt(0),d="/"===a.substr(-1);(a=$a(a.split("/").filter(function(f){return !!f}),!b).join("/"))||b||(a=".");a&&d&&(a+="/");return (b?"/":"")+a}
    function bb(a){var b=/^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/.exec(a).slice(1);a=b[0];b=b[1];if(!a&&!b)return ".";b&&(b=b.substr(0,b.length-1));return a+b}function T(a){if("/"===a)return "/";a=ab(a);a=a.replace(/\/$/,"");var b=a.lastIndexOf("/");return -1===b?a:a.substr(b+1)}function U(a,b){return ab(a+"/"+b)}
    function cb(){if("object"===typeof crypto&&"function"===typeof crypto.getRandomValues){var a=new Uint8Array(1);return function(){crypto.getRandomValues(a);return a[0]}}if(m)try{var b=require("crypto");return function(){return b.randomBytes(1)[0]}}catch(d){}return function(){F("randomDevice");}}
    function V(){for(var a="",b=!1,d=arguments.length-1;-1<=d&&!b;d--){b=0<=d?arguments[d]:W.cwd();if("string"!==typeof b)throw new TypeError("Arguments to path.resolve must be strings");if(!b)return "";a=b+"/"+a;b="/"===b.charAt(0);}a=$a(a.split("/").filter(function(f){return !!f}),!b).join("/");return (b?"/":"")+a||"."}
    function db(a,b){function d(l){for(var p=0;p<l.length&&""===l[p];p++);for(var t=l.length-1;0<=t&&""===l[t];t--);return p>t?[]:l.slice(p,t-p+1)}a=V(a).substr(1);b=V(b).substr(1);a=d(a.split("/"));b=d(b.split("/"));for(var f=Math.min(a.length,b.length),g=f,h=0;h<f;h++)if(a[h]!==b[h]){g=h;break}f=[];for(h=g;h<a.length;h++)f.push("..");f=f.concat(b.slice(g));return f.join("/")}var eb=[];function fb(a,b){eb[a]={input:[],output:[],dd:b};W.Sd(a,gb);}
    var gb={open:function(a){var b=eb[a.node.rdev];if(!b)throw new W.Fc(43);a.tty=b;a.seekable=!1;},close:function(a){a.tty.dd.flush(a.tty);},flush:function(a){a.tty.dd.flush(a.tty);},read:function(a,b,d,f){if(!a.tty||!a.tty.dd.ee)throw new W.Fc(60);for(var g=0,h=0;h<f;h++){try{var l=a.tty.dd.ee(a.tty);}catch(p){throw new W.Fc(29);}if(void 0===l&&0===g)throw new W.Fc(6);if(null===l||void 0===l)break;g++;b[d+h]=l;}g&&(a.node.timestamp=Date.now());return g},write:function(a,b,d,f){if(!a.tty||!a.tty.dd.Pd)throw new W.Fc(60);
    try{for(var g=0;g<f;g++)a.tty.dd.Pd(a.tty,b[d+g]);}catch(h){throw new W.Fc(29);}f&&(a.node.timestamp=Date.now());return g}},ib={ee:function(a){if(!a.input.length){var b=null;if(m){var d=Buffer.alloc(256),f=0;try{f=fs.readSync(process.stdin.fd,d,0,256,null);}catch(g){if(g.toString().includes("EOF"))f=0;else throw g;}0<f?b=d.slice(0,f).toString("utf-8"):b=null;}else "undefined"!=typeof window&&"function"==typeof window.prompt?(b=window.prompt("Input: "),null!==b&&(b+="\n")):"function"==typeof readline&&
    (b=readline(),null!==b&&(b+="\n"));if(!b)return null;a.input=hb(b,!0);}return a.input.shift()},Pd:function(a,b){null===b||10===b?(na(K(a.output,0)),a.output=[]):0!=b&&a.output.push(b);},flush:function(a){a.output&&0<a.output.length&&(na(K(a.output,0)),a.output=[]);}},jb={Pd:function(a,b){null===b||10===b?(A(K(a.output,0)),a.output=[]):0!=b&&a.output.push(b);},flush:function(a){a.output&&0<a.output.length&&(A(K(a.output,0)),a.output=[]);}},X={Uc:null,Lc:function(){return X.createNode(null,"/",16895,0)},
    createNode:function(a,b,d,f){if(W.Ee(d)||W.isFIFO(d))throw new W.Fc(63);X.Uc||(X.Uc={dir:{node:{Rc:X.Gc.Rc,Oc:X.Gc.Oc,lookup:X.Gc.lookup,Wc:X.Gc.Wc,rename:X.Gc.rename,unlink:X.Gc.unlink,rmdir:X.Gc.rmdir,readdir:X.Gc.readdir,symlink:X.Gc.symlink},stream:{Sc:X.Ic.Sc}},file:{node:{Rc:X.Gc.Rc,Oc:X.Gc.Oc},stream:{Sc:X.Ic.Sc,read:X.Ic.read,write:X.Ic.write,hd:X.Ic.hd,md:X.Ic.md,pd:X.Ic.pd}},link:{node:{Rc:X.Gc.Rc,Oc:X.Gc.Oc,readlink:X.Gc.readlink},stream:{}},Wd:{node:{Rc:X.Gc.Rc,Oc:X.Gc.Oc},stream:W.se}});
    d=W.createNode(a,b,d,f);W.Nc(d.mode)?(d.Gc=X.Uc.dir.node,d.Ic=X.Uc.dir.stream,d.Hc={}):W.isFile(d.mode)?(d.Gc=X.Uc.file.node,d.Ic=X.Uc.file.stream,d.Kc=0,d.Hc=null):W.ld(d.mode)?(d.Gc=X.Uc.link.node,d.Ic=X.Uc.link.stream):W.sd(d.mode)&&(d.Gc=X.Uc.Wd.node,d.Ic=X.Uc.Wd.stream);d.timestamp=Date.now();a&&(a.Hc[b]=d,a.timestamp=d.timestamp);return d},cf:function(a){return a.Hc?a.Hc.subarray?a.Hc.subarray(0,a.Kc):new Uint8Array(a.Hc):new Uint8Array(0)},ae:function(a,b){var d=a.Hc?a.Hc.length:0;d>=b||(b=
    Math.max(b,d*(1048576>d?2:1.125)>>>0),0!=d&&(b=Math.max(b,256)),d=a.Hc,a.Hc=new Uint8Array(b),0<a.Kc&&a.Hc.set(d.subarray(0,a.Kc),0));},Oe:function(a,b){if(a.Kc!=b)if(0==b)a.Hc=null,a.Kc=0;else {var d=a.Hc;a.Hc=new Uint8Array(b);d&&a.Hc.set(d.subarray(0,Math.min(b,a.Kc)));a.Kc=b;}},Gc:{Rc:function(a){var b={};b.dev=W.sd(a.mode)?a.id:1;b.ino=a.id;b.mode=a.mode;b.nlink=1;b.uid=0;b.gid=0;b.rdev=a.rdev;W.Nc(a.mode)?b.size=4096:W.isFile(a.mode)?b.size=a.Kc:W.ld(a.mode)?b.size=a.link.length:b.size=0;b.atime=
    new Date(a.timestamp);b.mtime=new Date(a.timestamp);b.ctime=new Date(a.timestamp);b.qe=4096;b.blocks=Math.ceil(b.size/b.qe);return b},Oc:function(a,b){void 0!==b.mode&&(a.mode=b.mode);void 0!==b.timestamp&&(a.timestamp=b.timestamp);void 0!==b.size&&X.Oe(a,b.size);},lookup:function(){throw W.Fd[44];},Wc:function(a,b,d,f){return X.createNode(a,b,d,f)},rename:function(a,b,d){if(W.Nc(a.mode)){try{var f=W.Vc(b,d);}catch(h){}if(f)for(var g in f.Hc)throw new W.Fc(55);}delete a.parent.Hc[a.name];a.parent.timestamp=
    Date.now();a.name=d;b.Hc[d]=a;b.timestamp=a.parent.timestamp;a.parent=b;},unlink:function(a,b){delete a.Hc[b];a.timestamp=Date.now();},rmdir:function(a,b){var d=W.Vc(a,b),f;for(f in d.Hc)throw new W.Fc(55);delete a.Hc[b];a.timestamp=Date.now();},readdir:function(a){var b=[".",".."],d;for(d in a.Hc)a.Hc.hasOwnProperty(d)&&b.push(d);return b},symlink:function(a,b,d){a=X.createNode(a,b,41471,0);a.link=d;return a},readlink:function(a){if(!W.ld(a.mode))throw new W.Fc(28);return a.link}},Ic:{read:function(a,
    b,d,f,g){var h=a.node.Hc;if(g>=a.node.Kc)return 0;a=Math.min(a.node.Kc-g,f);if(8<a&&h.subarray)b.set(h.subarray(g,g+a),d);else for(f=0;f<a;f++)b[d+f]=h[g+f];return a},write:function(a,b,d,f,g,h){b.buffer===H.buffer&&(h=!1);if(!f)return 0;a=a.node;a.timestamp=Date.now();if(b.subarray&&(!a.Hc||a.Hc.subarray)){if(h)return a.Hc=b.subarray(d,d+f),a.Kc=f;if(0===a.Kc&&0===g)return a.Hc=b.slice(d,d+f),a.Kc=f;if(g+f<=a.Kc)return a.Hc.set(b.subarray(d,d+f),g),f}X.ae(a,g+f);if(a.Hc.subarray&&b.subarray)a.Hc.set(b.subarray(d,
    d+f),g);else for(h=0;h<f;h++)a.Hc[g+h]=b[d+h];a.Kc=Math.max(a.Kc,g+f);return f},Sc:function(a,b,d){1===d?b+=a.position:2===d&&W.isFile(a.node.mode)&&(b+=a.node.Kc);if(0>b)throw new W.Fc(28);return b},hd:function(a,b,d){X.ae(a.node,b+d);a.node.Kc=Math.max(a.node.Kc,b+d);},md:function(a,b,d,f,g,h){if(0!==b)throw new W.Fc(28);if(!W.isFile(a.node.mode))throw new W.Fc(43);a=a.node.Hc;if(h&2||a.buffer!==Ba){if(0<f||f+d<a.length)a.subarray?a=a.subarray(f,f+d):a=Array.prototype.slice.call(a,f,f+d);f=!0;F();
    d=void 0;if(!d)throw new W.Fc(48);H.set(a,d);}else f=!1,d=a.byteOffset;return {jf:d,We:f}},pd:function(a,b,d,f,g){if(!W.isFile(a.node.mode))throw new W.Fc(43);if(g&2)return 0;X.Ic.write(a,b,0,f,d,!1);return 0}}};function kb(a,b,d){var f="al "+a;z(a,function(g){g||F('Loading data file "'+a+'" failed (no arrayBuffer).');b(new Uint8Array(g));f&&Na();},function(){if(d)d();else throw 'Loading data file "'+a+'" failed.';});f&&Ma();}
    var W={root:null,od:[],Zd:{},streams:[],Je:1,Tc:null,Yd:"/",Jd:!1,ie:!0,Fc:null,Fd:{},Be:null,wd:0,Jc:function(a,b){a=V(W.cwd(),a);b=b||{};if(!a)return {path:"",node:null};var d={Ed:!0,Rd:0},f;for(f in d)void 0===b[f]&&(b[f]=d[f]);if(8<b.Rd)throw new W.Fc(32);a=$a(a.split("/").filter(function(l){return !!l}),!1);var g=W.root;d="/";for(f=0;f<a.length;f++){var h=f===a.length-1;if(h&&b.parent)break;g=W.Vc(g,a[f]);d=U(d,a[f]);W.Zc(g)&&(!h||h&&b.Ed)&&(g=g.nd.root);if(!h||b.Qc)for(h=0;W.ld(g.mode);)if(g=
    W.readlink(d),d=V(bb(d),g),g=W.Jc(d,{Rd:b.Rd}).node,40<h++)throw new W.Fc(32);}return {path:d,node:g}},Yc:function(a){for(var b;;){if(W.td(a))return a=a.Lc.je,b?"/"!==a[a.length-1]?a+"/"+b:a+b:a;b=b?a.name+"/"+b:a.name;a=a.parent;}},Id:function(a,b){for(var d=0,f=0;f<b.length;f++)d=(d<<5)-d+b.charCodeAt(f)|0;return (a+d>>>0)%W.Tc.length},ge:function(a){var b=W.Id(a.parent.id,a.name);a.ad=W.Tc[b];W.Tc[b]=a;},he:function(a){var b=W.Id(a.parent.id,a.name);if(W.Tc[b]===a)W.Tc[b]=a.ad;else for(b=W.Tc[b];b;){if(b.ad===
    a){b.ad=a.ad;break}b=b.ad;}},Vc:function(a,b){var d=W.Ge(a);if(d)throw new W.Fc(d,a);for(d=W.Tc[W.Id(a.id,b)];d;d=d.ad){var f=d.name;if(d.parent.id===a.id&&f===b)return d}return W.lookup(a,b)},createNode:function(a,b,d,f){a=new W.le(a,b,d,f);W.ge(a);return a},Dd:function(a){W.he(a);},td:function(a){return a===a.parent},Zc:function(a){return !!a.nd},isFile:function(a){return 32768===(a&61440)},Nc:function(a){return 16384===(a&61440)},ld:function(a){return 40960===(a&61440)},sd:function(a){return 8192===
    (a&61440)},Ee:function(a){return 24576===(a&61440)},isFIFO:function(a){return 4096===(a&61440)},isSocket:function(a){return 49152===(a&49152)},Ce:{r:0,"r+":2,w:577,"w+":578,a:1089,"a+":1090},Ie:function(a){var b=W.Ce[a];if("undefined"===typeof b)throw Error("Unknown file open mode: "+a);return b},be:function(a){var b=["r","w","rw"][a&3];a&512&&(b+="w");return b},bd:function(a,b){if(W.ie)return 0;if(!b.includes("r")||a.mode&292){if(b.includes("w")&&!(a.mode&146)||b.includes("x")&&!(a.mode&73))return 2}else return 2;
    return 0},Ge:function(a){var b=W.bd(a,"x");return b?b:a.Gc.lookup?0:2},Od:function(a,b){try{return W.Vc(a,b),20}catch(d){}return W.bd(a,"wx")},ud:function(a,b,d){try{var f=W.Vc(a,b);}catch(g){return g.Mc}if(a=W.bd(a,"wx"))return a;if(d){if(!W.Nc(f.mode))return 54;if(W.td(f)||W.Yc(f)===W.cwd())return 10}else if(W.Nc(f.mode))return 31;return 0},He:function(a,b){return a?W.ld(a.mode)?32:W.Nc(a.mode)&&("r"!==W.be(b)||b&512)?31:W.bd(a,W.be(b)):44},ne:4096,Ke:function(a,b){b=b||W.ne;for(a=a||0;a<=b;a++)if(!W.streams[a])return a;
    throw new W.Fc(33);},cd:function(a){return W.streams[a]},ze:function(a,b,d){W.Ad||(W.Ad=function(){},W.Ad.prototype={object:{get:function(){return this.node},set:function(h){this.node=h;}}});var f=new W.Ad,g;for(g in a)f[g]=a[g];a=f;b=W.Ke(b,d);a.fd=b;return W.streams[b]=a},te:function(a){W.streams[a]=null;},se:{open:function(a){a.Ic=W.De(a.node.rdev).Ic;a.Ic.open&&a.Ic.open(a);},Sc:function(){throw new W.Fc(70);}},Nd:function(a){return a>>8},ef:function(a){return a&255},$c:function(a,b){return a<<8|
    b},Sd:function(a,b){W.Zd[a]={Ic:b};},De:function(a){return W.Zd[a]},de:function(a){var b=[];for(a=[a];a.length;){var d=a.pop();b.push(d);a.push.apply(a,d.od);}return b},ke:function(a,b){function d(l){W.wd--;return b(l)}function f(l){if(l){if(!f.Ae)return f.Ae=!0,d(l)}else ++h>=g.length&&d(null);}"function"===typeof a&&(b=a,a=!1);W.wd++;1<W.wd&&A("warning: "+W.wd+" FS.syncfs operations in flight at once, probably just doing extra work");var g=W.de(W.root.Lc),h=0;g.forEach(function(l){if(!l.type.ke)return f(null);
    l.type.ke(l,a,f);});},Lc:function(a,b,d){var f="/"===d,g=!d;if(f&&W.root)throw new W.Fc(10);if(!f&&!g){var h=W.Jc(d,{Ed:!1});d=h.path;h=h.node;if(W.Zc(h))throw new W.Fc(10);if(!W.Nc(h.mode))throw new W.Fc(54);}b={type:a,hf:b,je:d,od:[]};a=a.Lc(b);a.Lc=b;b.root=a;f?W.root=a:h&&(h.nd=b,h.Lc&&h.Lc.od.push(b));return a},qf:function(a){a=W.Jc(a,{Ed:!1});if(!W.Zc(a.node))throw new W.Fc(28);a=a.node;var b=a.nd,d=W.de(b);Object.keys(W.Tc).forEach(function(f){for(f=W.Tc[f];f;){var g=f.ad;d.includes(f.Lc)&&W.Dd(f);
    f=g;}});a.nd=null;a.Lc.od.splice(a.Lc.od.indexOf(b),1);},lookup:function(a,b){return a.Gc.lookup(a,b)},Wc:function(a,b,d){var f=W.Jc(a,{parent:!0}).node;a=T(a);if(!a||"."===a||".."===a)throw new W.Fc(28);var g=W.Od(f,a);if(g)throw new W.Fc(g);if(!f.Gc.Wc)throw new W.Fc(63);return f.Gc.Wc(f,a,b,d)},create:function(a,b){return W.Wc(a,(void 0!==b?b:438)&4095|32768,0)},mkdir:function(a,b){return W.Wc(a,(void 0!==b?b:511)&1023|16384,0)},ff:function(a,b){a=a.split("/");for(var d="",f=0;f<a.length;++f)if(a[f]){d+=
    "/"+a[f];try{W.mkdir(d,b);}catch(g){if(20!=g.Mc)throw g;}}},vd:function(a,b,d){"undefined"===typeof d&&(d=b,b=438);return W.Wc(a,b|8192,d)},symlink:function(a,b){if(!V(a))throw new W.Fc(44);var d=W.Jc(b,{parent:!0}).node;if(!d)throw new W.Fc(44);b=T(b);var f=W.Od(d,b);if(f)throw new W.Fc(f);if(!d.Gc.symlink)throw new W.Fc(63);return d.Gc.symlink(d,b,a)},rename:function(a,b){var d=bb(a),f=bb(b),g=T(a),h=T(b);var l=W.Jc(a,{parent:!0});var p=l.node;l=W.Jc(b,{parent:!0});l=l.node;if(!p||!l)throw new W.Fc(44);
    if(p.Lc!==l.Lc)throw new W.Fc(75);var t=W.Vc(p,g);a=db(a,f);if("."!==a.charAt(0))throw new W.Fc(28);a=db(b,d);if("."!==a.charAt(0))throw new W.Fc(55);try{var n=W.Vc(l,h);}catch(q){}if(t!==n){b=W.Nc(t.mode);if(g=W.ud(p,g,b))throw new W.Fc(g);if(g=n?W.ud(l,h,b):W.Od(l,h))throw new W.Fc(g);if(!p.Gc.rename)throw new W.Fc(63);if(W.Zc(t)||n&&W.Zc(n))throw new W.Fc(10);if(l!==p&&(g=W.bd(p,"w")))throw new W.Fc(g);W.he(t);try{p.Gc.rename(t,l,h);}catch(q){throw q;}finally{W.ge(t);}}},rmdir:function(a){var b=W.Jc(a,
    {parent:!0}).node;a=T(a);var d=W.Vc(b,a),f=W.ud(b,a,!0);if(f)throw new W.Fc(f);if(!b.Gc.rmdir)throw new W.Fc(63);if(W.Zc(d))throw new W.Fc(10);b.Gc.rmdir(b,a);W.Dd(d);},readdir:function(a){a=W.Jc(a,{Qc:!0}).node;if(!a.Gc.readdir)throw new W.Fc(54);return a.Gc.readdir(a)},unlink:function(a){var b=W.Jc(a,{parent:!0}).node;if(!b)throw new W.Fc(44);a=T(a);var d=W.Vc(b,a),f=W.ud(b,a,!1);if(f)throw new W.Fc(f);if(!b.Gc.unlink)throw new W.Fc(63);if(W.Zc(d))throw new W.Fc(10);b.Gc.unlink(b,a);W.Dd(d);},readlink:function(a){a=
    W.Jc(a).node;if(!a)throw new W.Fc(44);if(!a.Gc.readlink)throw new W.Fc(28);return V(W.Yc(a.parent),a.Gc.readlink(a))},stat:function(a,b){a=W.Jc(a,{Qc:!b}).node;if(!a)throw new W.Fc(44);if(!a.Gc.Rc)throw new W.Fc(63);return a.Gc.Rc(a)},lstat:function(a){return W.stat(a,!0)},chmod:function(a,b,d){a="string"===typeof a?W.Jc(a,{Qc:!d}).node:a;if(!a.Gc.Oc)throw new W.Fc(63);a.Gc.Oc(a,{mode:b&4095|a.mode&-4096,timestamp:Date.now()});},lchmod:function(a,b){W.chmod(a,b,!0);},fchmod:function(a,b){a=W.cd(a);
    if(!a)throw new W.Fc(8);W.chmod(a.node,b);},chown:function(a,b,d,f){a="string"===typeof a?W.Jc(a,{Qc:!f}).node:a;if(!a.Gc.Oc)throw new W.Fc(63);a.Gc.Oc(a,{timestamp:Date.now()});},lchown:function(a,b,d){W.chown(a,b,d,!0);},fchown:function(a,b,d){a=W.cd(a);if(!a)throw new W.Fc(8);W.chown(a.node,b,d);},truncate:function(a,b){if(0>b)throw new W.Fc(28);a="string"===typeof a?W.Jc(a,{Qc:!0}).node:a;if(!a.Gc.Oc)throw new W.Fc(63);if(W.Nc(a.mode))throw new W.Fc(31);if(!W.isFile(a.mode))throw new W.Fc(28);var d=
    W.bd(a,"w");if(d)throw new W.Fc(d);a.Gc.Oc(a,{size:b,timestamp:Date.now()});},af:function(a,b){a=W.cd(a);if(!a)throw new W.Fc(8);if(0===(a.flags&2097155))throw new W.Fc(28);W.truncate(a.node,b);},rf:function(a,b,d){a=W.Jc(a,{Qc:!0}).node;a.Gc.Oc(a,{timestamp:Math.max(b,d)});},open:function(a,b,d,f,g){if(""===a)throw new W.Fc(44);b="string"===typeof b?W.Ie(b):b;d=b&64?("undefined"===typeof d?438:d)&4095|32768:0;if("object"===typeof a)var h=a;else {a=ab(a);try{h=W.Jc(a,{Qc:!(b&131072)}).node;}catch(p){}}var l=
    !1;if(b&64)if(h){if(b&128)throw new W.Fc(20);}else h=W.Wc(a,d,0),l=!0;if(!h)throw new W.Fc(44);W.sd(h.mode)&&(b&=-513);if(b&65536&&!W.Nc(h.mode))throw new W.Fc(54);if(!l&&(d=W.He(h,b)))throw new W.Fc(d);b&512&&W.truncate(h,0);b&=-131713;f=W.ze({node:h,path:W.Yc(h),id:h.id,flags:b,mode:h.mode,seekable:!0,position:0,Ic:h.Ic,Gc:h.Gc,Ve:[],error:!1},f,g);f.Ic.open&&f.Ic.open(f);!c.logReadFiles||b&1||(W.Qd||(W.Qd={}),a in W.Qd||(W.Qd[a]=1));return f},close:function(a){if(W.kd(a))throw new W.Fc(8);a.Hd&&
    (a.Hd=null);try{a.Ic.close&&a.Ic.close(a);}catch(b){throw b;}finally{W.te(a.fd);}a.fd=null;},kd:function(a){return null===a.fd},Sc:function(a,b,d){if(W.kd(a))throw new W.Fc(8);if(!a.seekable||!a.Ic.Sc)throw new W.Fc(70);if(0!=d&&1!=d&&2!=d)throw new W.Fc(28);a.position=a.Ic.Sc(a,b,d);a.Ve=[];return a.position},read:function(a,b,d,f,g){if(0>f||0>g)throw new W.Fc(28);if(W.kd(a))throw new W.Fc(8);if(1===(a.flags&2097155))throw new W.Fc(8);if(W.Nc(a.node.mode))throw new W.Fc(31);if(!a.Ic.read)throw new W.Fc(28);
    var h="undefined"!==typeof g;if(!h)g=a.position;else if(!a.seekable)throw new W.Fc(70);b=a.Ic.read(a,b,d,f,g);h||(a.position+=b);return b},write:function(a,b,d,f,g,h){if(0>f||0>g)throw new W.Fc(28);if(W.kd(a))throw new W.Fc(8);if(0===(a.flags&2097155))throw new W.Fc(8);if(W.Nc(a.node.mode))throw new W.Fc(31);if(!a.Ic.write)throw new W.Fc(28);a.seekable&&a.flags&1024&&W.Sc(a,0,2);var l="undefined"!==typeof g;if(!l)g=a.position;else if(!a.seekable)throw new W.Fc(70);b=a.Ic.write(a,b,d,f,g,h);l||(a.position+=
    b);return b},hd:function(a,b,d){if(W.kd(a))throw new W.Fc(8);if(0>b||0>=d)throw new W.Fc(28);if(0===(a.flags&2097155))throw new W.Fc(8);if(!W.isFile(a.node.mode)&&!W.Nc(a.node.mode))throw new W.Fc(43);if(!a.Ic.hd)throw new W.Fc(138);a.Ic.hd(a,b,d);},md:function(a,b,d,f,g,h){if(0!==(g&2)&&0===(h&2)&&2!==(a.flags&2097155))throw new W.Fc(2);if(1===(a.flags&2097155))throw new W.Fc(2);if(!a.Ic.md)throw new W.Fc(43);return a.Ic.md(a,b,d,f,g,h)},pd:function(a,b,d,f,g){return a&&a.Ic.pd?a.Ic.pd(a,b,d,f,g):
    0},gf:function(){return 0},Kd:function(a,b,d){if(!a.Ic.Kd)throw new W.Fc(59);return a.Ic.Kd(a,b,d)},readFile:function(a,b){b=b||{};b.flags=b.flags||0;b.encoding=b.encoding||"binary";if("utf8"!==b.encoding&&"binary"!==b.encoding)throw Error('Invalid encoding type "'+b.encoding+'"');var d,f=W.open(a,b.flags);a=W.stat(a).size;var g=new Uint8Array(a);W.read(f,g,0,a,0);"utf8"===b.encoding?d=K(g,0):"binary"===b.encoding&&(d=g);W.close(f);return d},writeFile:function(a,b,d){d=d||{};d.flags=d.flags||577;
    a=W.open(a,d.flags,d.mode);if("string"===typeof b){var f=new Uint8Array(ya(b)+1);b=ua(b,f,0,f.length);W.write(a,f,0,b,void 0,d.re);}else if(ArrayBuffer.isView(b))W.write(a,b,0,b.byteLength,void 0,d.re);else throw Error("Unsupported data type");W.close(a);},cwd:function(){return W.Yd},chdir:function(a){a=W.Jc(a,{Qc:!0});if(null===a.node)throw new W.Fc(44);if(!W.Nc(a.node.mode))throw new W.Fc(54);var b=W.bd(a.node,"x");if(b)throw new W.Fc(b);W.Yd=a.path;},ve:function(){W.mkdir("/tmp");W.mkdir("/home");
    W.mkdir("/home/web_user");},ue:function(){W.mkdir("/dev");W.Sd(W.$c(1,3),{read:function(){return 0},write:function(b,d,f,g){return g}});W.vd("/dev/null",W.$c(1,3));fb(W.$c(5,0),ib);fb(W.$c(6,0),jb);W.vd("/dev/tty",W.$c(5,0));W.vd("/dev/tty1",W.$c(6,0));var a=cb();W.Xc("/dev","random",a);W.Xc("/dev","urandom",a);W.mkdir("/dev/shm");W.mkdir("/dev/shm/tmp");},xe:function(){W.mkdir("/proc");var a=W.mkdir("/proc/self");W.mkdir("/proc/self/fd");W.Lc({Lc:function(){var b=W.createNode(a,"fd",16895,73);b.Gc=
    {lookup:function(d,f){var g=W.cd(+f);if(!g)throw new W.Fc(8);d={parent:null,Lc:{je:"fake"},Gc:{readlink:function(){return g.path}}};return d.parent=d}};return b}},{},"/proc/self/fd");},ye:function(){c.stdin?W.Xc("/dev","stdin",c.stdin):W.symlink("/dev/tty","/dev/stdin");c.stdout?W.Xc("/dev","stdout",null,c.stdout):W.symlink("/dev/tty","/dev/stdout");c.stderr?W.Xc("/dev","stderr",null,c.stderr):W.symlink("/dev/tty1","/dev/stderr");W.open("/dev/stdin",0);W.open("/dev/stdout",1);W.open("/dev/stderr",
    1);},$d:function(){W.Fc||(W.Fc=function(a,b){this.node=b;this.Pe=function(d){this.Mc=d;};this.Pe(a);this.message="FS error";},W.Fc.prototype=Error(),W.Fc.prototype.constructor=W.Fc,[44].forEach(function(a){W.Fd[a]=new W.Fc(a);W.Fd[a].stack="<generic error, no stack>";}));},Qe:function(){W.$d();W.Tc=Array(4096);W.Lc(X,{},"/");W.ve();W.ue();W.xe();W.Be={MEMFS:X};},jd:function(a,b,d){W.jd.Jd=!0;W.$d();c.stdin=a||c.stdin;c.stdout=b||c.stdout;c.stderr=d||c.stderr;W.ye();},kf:function(){W.jd.Jd=!1;for(var a=0;a<
    W.streams.length;a++){var b=W.streams[a];b&&W.close(b);}},Gd:function(a,b){var d=0;a&&(d|=365);b&&(d|=146);return d},$e:function(a,b){a=W.Bd(a,b);return a.exists?a.object:null},Bd:function(a,b){try{var d=W.Jc(a,{Qc:!b});a=d.path;}catch(g){}var f={td:!1,exists:!1,error:0,name:null,path:null,object:null,Le:!1,Ne:null,Me:null};try{d=W.Jc(a,{parent:!0}),f.Le=!0,f.Ne=d.path,f.Me=d.node,f.name=T(a),d=W.Jc(a,{Qc:!b}),f.exists=!0,f.path=d.path,f.object=d.node,f.name=d.node.name,f.td="/"===d.path;}catch(g){f.error=
    g.Mc;}return f},Ye:function(a,b){a="string"===typeof a?a:W.Yc(a);for(b=b.split("/").reverse();b.length;){var d=b.pop();if(d){var f=U(a,d);try{W.mkdir(f);}catch(g){}a=f;}}return f},we:function(a,b,d,f,g){a=U("string"===typeof a?a:W.Yc(a),b);return W.create(a,W.Gd(f,g))},Xd:function(a,b,d,f,g,h){a=b?U("string"===typeof a?a:W.Yc(a),b):a;f=W.Gd(f,g);g=W.create(a,f);if(d){if("string"===typeof d){a=Array(d.length);b=0;for(var l=d.length;b<l;++b)a[b]=d.charCodeAt(b);d=a;}W.chmod(g,f|146);a=W.open(g,577);W.write(a,
    d,0,d.length,0,h);W.close(a);W.chmod(g,f);}return g},Xc:function(a,b,d,f){a=U("string"===typeof a?a:W.Yc(a),b);b=W.Gd(!!d,!!f);W.Xc.Nd||(W.Xc.Nd=64);var g=W.$c(W.Xc.Nd++,0);W.Sd(g,{open:function(h){h.seekable=!1;},close:function(){f&&f.buffer&&f.buffer.length&&f(10);},read:function(h,l,p,t){for(var n=0,q=0;q<t;q++){try{var w=d();}catch(y){throw new W.Fc(29);}if(void 0===w&&0===n)throw new W.Fc(6);if(null===w||void 0===w)break;n++;l[p+q]=w;}n&&(h.node.timestamp=Date.now());return n},write:function(h,l,
    p,t){for(var n=0;n<t;n++)try{f(l[p+n]);}catch(q){throw new W.Fc(29);}t&&(h.node.timestamp=Date.now());return n}});return W.vd(a,b,g)},ce:function(a){if(a.Ld||a.Fe||a.link||a.Hc)return !0;if("undefined"!==typeof XMLHttpRequest)throw Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");if(x)try{a.Hc=hb(x(a.url),!0),a.Kc=a.Hc.length;}catch(b){throw new W.Fc(29);
    }else throw Error("Cannot load without read() or XMLHttpRequest.");},Xe:function(a,b,d,f,g){function h(){this.Md=!1;this.rd=[];}h.prototype.get=function(n){if(!(n>this.length-1||0>n)){var q=n%this.chunkSize;return this.fe(n/this.chunkSize|0)[q]}};h.prototype.me=function(n){this.fe=n;};h.prototype.Vd=function(){var n=new XMLHttpRequest;n.open("HEAD",d,!1);n.send(null);if(!(200<=n.status&&300>n.status||304===n.status))throw Error("Couldn't load "+d+". Status: "+n.status);var q=Number(n.getResponseHeader("Content-length")),
    w,y=(w=n.getResponseHeader("Accept-Ranges"))&&"bytes"===w;n=(w=n.getResponseHeader("Content-Encoding"))&&"gzip"===w;var k=1048576;y||(k=q);var r=this;r.me(function(v){var B=v*k,I=(v+1)*k-1;I=Math.min(I,q-1);if("undefined"===typeof r.rd[v]){var Ab=r.rd;if(B>I)throw Error("invalid range ("+B+", "+I+") or no bytes requested!");if(I>q-1)throw Error("only "+q+" bytes available! programmer error!");var C=new XMLHttpRequest;C.open("GET",d,!1);q!==k&&C.setRequestHeader("Range","bytes="+B+"-"+I);"undefined"!=
    typeof Uint8Array&&(C.responseType="arraybuffer");C.overrideMimeType&&C.overrideMimeType("text/plain; charset=x-user-defined");C.send(null);if(!(200<=C.status&&300>C.status||304===C.status))throw Error("Couldn't load "+d+". Status: "+C.status);B=void 0!==C.response?new Uint8Array(C.response||[]):hb(C.responseText||"",!0);Ab[v]=B;}if("undefined"===typeof r.rd[v])throw Error("doXHR failed!");return r.rd[v]});if(n||!q)k=q=1,k=q=this.fe(0).length,na("LazyFiles on gzip forces download of the whole file when length is accessed");
    this.pe=q;this.oe=k;this.Md=!0;};if("undefined"!==typeof XMLHttpRequest){if(!e)throw "Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc";var l=new h;Object.defineProperties(l,{length:{get:function(){this.Md||this.Vd();return this.pe}},chunkSize:{get:function(){this.Md||this.Vd();return this.oe}}});l={Ld:!1,Hc:l};}else l={Ld:!1,url:d};var p=W.we(a,b,l,f,g);l.Hc?p.Hc=l.Hc:l.url&&(p.Hc=null,p.url=l.url);Object.defineProperties(p,{Kc:{get:function(){return this.Hc.length}}});
    var t={};Object.keys(p.Ic).forEach(function(n){var q=p.Ic[n];t[n]=function(){W.ce(p);return q.apply(null,arguments)};});t.read=function(n,q,w,y,k){W.ce(p);n=n.node.Hc;if(k>=n.length)return 0;y=Math.min(n.length-k,y);if(n.slice)for(var r=0;r<y;r++)q[w+r]=n[k+r];else for(r=0;r<y;r++)q[w+r]=n.get(k+r);return y};p.Ic=t;return p},Ze:function(a,b,d,f,g,h,l,p,t,n){function q(y){function k(v){n&&n();p||W.Xd(a,b,v,f,g,t);h&&h();Na();}var r=!1;c.preloadPlugins.forEach(function(v){!r&&v.canHandle(w)&&(v.handle(y,
    w,k,function(){l&&l();Na();}),r=!0);});r||k(y);}lb.jd();var w=b?V(U(a,b)):a;Ma();"string"==typeof d?kb(d,function(y){q(y);},l):q(d);},indexedDB:function(){return window.indexedDB||window.mozIndexedDB||window.webkitIndexedDB||window.msIndexedDB},Td:function(){return "EM_FS_"+window.location.pathname},Ud:20,gd:"FILE_DATA",lf:function(a,b,d){b=b||function(){};d=d||function(){};var f=W.indexedDB();try{var g=f.open(W.Td(),W.Ud);}catch(h){return d(h)}g.onupgradeneeded=function(){na("creating db");g.result.createObjectStore(W.gd);};
    g.onsuccess=function(){var h=g.result.transaction([W.gd],"readwrite"),l=h.objectStore(W.gd),p=0,t=0,n=a.length;a.forEach(function(q){q=l.put(W.Bd(q).object.Hc,q);q.onsuccess=function(){p++;p+t==n&&(0==t?b():d());};q.onerror=function(){t++;p+t==n&&(0==t?b():d());};});h.onerror=d;};g.onerror=d;},df:function(a,b,d){b=b||function(){};d=d||function(){};var f=W.indexedDB();try{var g=f.open(W.Td(),W.Ud);}catch(h){return d(h)}g.onupgradeneeded=d;g.onsuccess=function(){var h=g.result;try{var l=h.transaction([W.gd],
    "readonly");}catch(w){d(w);return}var p=l.objectStore(W.gd),t=0,n=0,q=a.length;a.forEach(function(w){var y=p.get(w);y.onsuccess=function(){W.Bd(w).exists&&W.unlink(w);W.Xd(bb(w),T(w),y.result,!0,!0,!0);t++;t+n==q&&(0==n?b():d());};y.onerror=function(){n++;t+n==q&&(0==n?b():d());};});l.onerror=d;};g.onerror=d;}},mb=void 0;function Y(){mb+=4;return L[mb-4>>2]}function Z(a){a=W.cd(a);if(!a)throw new W.Fc(8);return a}
    function nb(){void 0===nb.start&&(nb.start=Date.now());return 1E3*(Date.now()-nb.start)|0}var ob={};function pb(){if(!qb){var a={USER:"web_user",LOGNAME:"web_user",PATH:"/",PWD:"/",HOME:"/home/web_user",LANG:("object"===typeof navigator&&navigator.languages&&navigator.languages[0]||"C").replace("-","_")+".UTF-8",_:ea||"./this.program"},b;for(b in ob)void 0===ob[b]?delete a[b]:a[b]=ob[b];var d=[];for(b in a)d.push(b+"="+a[b]);qb=d;}return qb}var qb;
    function rb(a){return 0===a%4&&(0!==a%100||0===a%400)}function sb(a,b){for(var d=0,f=0;f<=b;d+=a[f++]);return d}var tb=[31,29,31,30,31,30,31,31,30,31,30,31],ub=[31,28,31,30,31,30,31,31,30,31,30,31];function vb(a,b){for(a=new Date(a.getTime());0<b;){var d=a.getMonth(),f=(rb(a.getFullYear())?tb:ub)[d];if(b>f-a.getDate())b-=f-a.getDate()+1,a.setDate(1),11>d?a.setMonth(d+1):(a.setMonth(0),a.setFullYear(a.getFullYear()+1));else {a.setDate(a.getDate()+b);break}}return a}
    function wb(a,b,d,f){a||(a=this);this.parent=a;this.Lc=a.Lc;this.nd=null;this.id=W.Je++;this.name=b;this.mode=d;this.Gc={};this.Ic={};this.rdev=f;}Object.defineProperties(wb.prototype,{read:{get:function(){return 365===(this.mode&365)},set:function(a){a?this.mode|=365:this.mode&=-366;}},write:{get:function(){return 146===(this.mode&146)},set:function(a){a?this.mode|=146:this.mode&=-147;}},Fe:{get:function(){return W.Nc(this.mode)}},Ld:{get:function(){return W.sd(this.mode)}}});W.le=wb;W.Qe();var lb;
    function hb(a,b){var d=Array(ya(a)+1);a=ua(a,d,0,d.length);b&&(d.length=a);return d}
    var zb={s:function(a,b){a:{if(0===a)a=Date.now();else if(1===a||4===a)a=Ua();else {L[xb()>>2]=28;b=-1;break a}L[b>>2]=a/1E3|0;L[b+4>>2]=a%1E3*1E6|0;b=0;}return b},w:function(a,b){a=new Date(1E3*L[a>>2]);L[b>>2]=a.getUTCSeconds();L[b+4>>2]=a.getUTCMinutes();L[b+8>>2]=a.getUTCHours();L[b+12>>2]=a.getUTCDate();L[b+16>>2]=a.getUTCMonth();L[b+20>>2]=a.getUTCFullYear()-1900;L[b+24>>2]=a.getUTCDay();L[b+36>>2]=0;L[b+32>>2]=0;L[b+28>>2]=(a.getTime()-Date.UTC(a.getUTCFullYear(),0,1,0,0,0,0))/864E5|0;Va||(Va=
    za("GMT"));L[b+40>>2]=Va;return b},v:function(a,b){Za||(Za=!0,Wa());a=new Date(1E3*L[a>>2]);L[b>>2]=a.getSeconds();L[b+4>>2]=a.getMinutes();L[b+8>>2]=a.getHours();L[b+12>>2]=a.getDate();L[b+16>>2]=a.getMonth();L[b+20>>2]=a.getFullYear()-1900;L[b+24>>2]=a.getDay();var d=new Date(a.getFullYear(),0,1);L[b+28>>2]=(a.getTime()-d.getTime())/864E5|0;L[b+36>>2]=-(60*a.getTimezoneOffset());var f=(new Date(a.getFullYear(),6,1)).getTimezoneOffset();d=d.getTimezoneOffset();a=(f!=d&&a.getTimezoneOffset()==Math.min(d,
    f))|0;L[b+32>>2]=a;a=L[S()+(a?4:0)>>2];L[b+40>>2]=a;return b},A:function(a,b){try{var d=Z(a);if(d.fd===b)var f=-28;else {var g=d.path,h=d.flags,l=W.cd(b);l&&W.close(l);f=W.open(g,h,0,b,b).fd;}return f}catch(p){if("undefined"===typeof W||!(p instanceof W.Fc))throw p;return -p.Mc}},d:function(a,b,d){mb=d;try{var f=Z(a);switch(b){case 0:var g=Y();return 0>g?-28:W.open(f.path,f.flags,0,g).fd;case 1:case 2:return 0;case 3:return f.flags;case 4:return g=Y(),f.flags|=g,0;case 5:return g=Y(),Ca[g+0>>1]=2,0;
    case 6:case 7:return 0;case 16:case 8:return -28;case 9:return L[xb()>>2]=28,-1;default:return -28}}catch(h){if("undefined"===typeof W||!(h instanceof W.Fc))throw h;return -h.Mc}},C:function(a,b,d){mb=d;try{var f=Z(a);switch(b){case 21509:case 21505:return f.tty?0:-59;case 21510:case 21511:case 21512:case 21506:case 21507:case 21508:return f.tty?0:-59;case 21519:if(!f.tty)return -59;var g=Y();return L[g>>2]=0;case 21520:return f.tty?-28:-59;case 21531:return g=Y(),W.Kd(f,b,g);case 21523:return f.tty?
    0:-59;case 21524:return f.tty?0:-59;default:F("bad ioctl syscall "+b);}}catch(h){if("undefined"===typeof W||!(h instanceof W.Fc))throw h;return -h.Mc}},r:function(a,b){try{a=J(a);a:{var d=W.lstat;try{var f=d(a);}catch(h){if(h&&h.node&&ab(a)!==ab(W.Yc(h.node))){var g=-54;break a}throw h;}L[b>>2]=f.dev;L[b+4>>2]=0;L[b+8>>2]=f.ino;L[b+12>>2]=f.mode;L[b+16>>2]=f.nlink;L[b+20>>2]=f.uid;L[b+24>>2]=f.gid;L[b+28>>2]=f.rdev;L[b+32>>2]=0;Q=[f.size>>>0,(P=f.size,1<=+Math.abs(P)?0<P?(Math.min(+Math.floor(P/4294967296),
    4294967295)|0)>>>0:~~+Math.ceil((P-+(~~P>>>0))/4294967296)>>>0:0)];L[b+40>>2]=Q[0];L[b+44>>2]=Q[1];L[b+48>>2]=4096;L[b+52>>2]=f.blocks;L[b+56>>2]=f.atime.getTime()/1E3|0;L[b+60>>2]=0;L[b+64>>2]=f.mtime.getTime()/1E3|0;L[b+68>>2]=0;L[b+72>>2]=f.ctime.getTime()/1E3|0;L[b+76>>2]=0;Q=[f.ino>>>0,(P=f.ino,1<=+Math.abs(P)?0<P?(Math.min(+Math.floor(P/4294967296),4294967295)|0)>>>0:~~+Math.ceil((P-+(~~P>>>0))/4294967296)>>>0:0)];L[b+80>>2]=Q[0];L[b+84>>2]=Q[1];g=0;}return g}catch(h){if("undefined"===typeof W||
    !(h instanceof W.Fc))throw h;return -h.Mc}},h:function(a,b,d){mb=d;try{var f=J(a),g=d?Y():0;return W.open(f,b,g).fd}catch(h){if("undefined"===typeof W||!(h instanceof W.Fc))throw h;return -h.Mc}},t:function(a,b){try{return a=J(a),b=J(b),W.rename(a,b),0}catch(d){if("undefined"===typeof W||!(d instanceof W.Fc))throw d;return -d.Mc}},u:function(a){try{return a=J(a),W.rmdir(a),0}catch(b){if("undefined"===typeof W||!(b instanceof W.Fc))throw b;return -b.Mc}},f:function(a){try{return a=J(a),W.unlink(a),0}catch(b){if("undefined"===
    typeof W||!(b instanceof W.Fc))throw b;return -b.Mc}},q:function(){throw "longjmp";},E:function(){F("");},e:nb,l:function(a,b){return a-b},o:function(a,b,d){G.copyWithin(a,b,b+d);},p:function(a){var b=G.length;a>>>=0;if(2147483648<a)return !1;for(var d=1;4>=d;d*=2){var f=b*(1+.2/d);f=Math.min(f,a+100663296);f=Math.max(a,f);0<f%65536&&(f+=65536-f%65536);a:{try{qa.grow(Math.min(2147483648,f)-Ba.byteLength+65535>>>16);Fa();var g=1;break a}catch(h){}g=void 0;}if(g)return !0}return !1},y:function(a,b){var d=0;
    pb().forEach(function(f,g){var h=b+d;g=L[a+4*g>>2]=h;for(h=0;h<f.length;++h)H[g++>>0]=f.charCodeAt(h);H[g>>0]=0;d+=f.length+1;});return 0},z:function(a,b){var d=pb();L[a>>2]=d.length;var f=0;d.forEach(function(g){f+=g.length+1;});L[b>>2]=f;return 0},j:function(a){if(!(noExitRuntime||0<la)){if(c.onExit)c.onExit(a);ra=!0;}fa(a,new ma(a));},c:function(a){try{var b=Z(a);W.close(b);return 0}catch(d){if("undefined"===typeof W||!(d instanceof W.Fc))throw d;return d.Mc}},B:function(a,b,d,f){try{a:{for(var g=
    Z(a),h=a=0;h<d;h++){var l=L[b+(8*h+4)>>2],p=W.read(g,H,L[b+8*h>>2],l,void 0);if(0>p){var t=-1;break a}a+=p;if(p<l)break}t=a;}L[f>>2]=t;return 0}catch(n){if("undefined"===typeof W||!(n instanceof W.Fc))throw n;return n.Mc}},n:function(a,b,d,f,g){try{var h=Z(a);a=4294967296*d+(b>>>0);if(-9007199254740992>=a||9007199254740992<=a)return -61;W.Sc(h,a,f);Q=[h.position>>>0,(P=h.position,1<=+Math.abs(P)?0<P?(Math.min(+Math.floor(P/4294967296),4294967295)|0)>>>0:~~+Math.ceil((P-+(~~P>>>0))/4294967296)>>>0:0)];
    L[g>>2]=Q[0];L[g+4>>2]=Q[1];h.Hd&&0===a&&0===f&&(h.Hd=null);return 0}catch(l){if("undefined"===typeof W||!(l instanceof W.Fc))throw l;return l.Mc}},g:function(a,b,d,f){try{a:{for(var g=Z(a),h=a=0;h<d;h++){var l=W.write(g,H,L[b+8*h>>2],L[b+(8*h+4)>>2],void 0);if(0>l){var p=-1;break a}a+=l;}p=a;}L[f>>2]=p;return 0}catch(t){if("undefined"===typeof W||!(t instanceof W.Fc))throw t;return t.Mc}},i:function(){return pa},x:yb,D:function(a){Za||(Za=!0,Wa());var b=new Date(L[a+20>>2]+1900,L[a+16>>2],L[a+12>>
    2],L[a+8>>2],L[a+4>>2],L[a>>2],0),d=L[a+32>>2],f=b.getTimezoneOffset(),g=new Date(b.getFullYear(),0,1),h=(new Date(b.getFullYear(),6,1)).getTimezoneOffset(),l=g.getTimezoneOffset(),p=Math.min(l,h);0>d?L[a+32>>2]=Number(h!=l&&p==f):0<d!=(p==f)&&(h=Math.max(l,h),b.setTime(b.getTime()+6E4*((0<d?p:h)-f)));L[a+24>>2]=b.getDay();L[a+28>>2]=(b.getTime()-g.getTime())/864E5|0;L[a>>2]=b.getSeconds();L[a+4>>2]=b.getMinutes();L[a+8>>2]=b.getHours();L[a+12>>2]=b.getDate();L[a+16>>2]=b.getMonth();return b.getTime()/
    1E3|0},a:function(a){pa=a;},m:function(a,b,d,f){function g(k,r,v){for(k="number"===typeof k?k.toString():k||"";k.length<r;)k=v[0]+k;return k}function h(k,r){return g(k,r,"0")}function l(k,r){function v(I){return 0>I?-1:0<I?1:0}var B;0===(B=v(k.getFullYear()-r.getFullYear()))&&0===(B=v(k.getMonth()-r.getMonth()))&&(B=v(k.getDate()-r.getDate()));return B}function p(k){switch(k.getDay()){case 0:return new Date(k.getFullYear()-1,11,29);case 1:return k;case 2:return new Date(k.getFullYear(),0,3);case 3:return new Date(k.getFullYear(),
    0,2);case 4:return new Date(k.getFullYear(),0,1);case 5:return new Date(k.getFullYear()-1,11,31);case 6:return new Date(k.getFullYear()-1,11,30)}}function t(k){k=vb(new Date(k.Pc+1900,0,1),k.zd);var r=new Date(k.getFullYear()+1,0,4),v=p(new Date(k.getFullYear(),0,4));r=p(r);return 0>=l(v,k)?0>=l(r,k)?k.getFullYear()+1:k.getFullYear():k.getFullYear()-1}var n=L[f+40>>2];f={Te:L[f>>2],Se:L[f+4>>2],xd:L[f+8>>2],qd:L[f+12>>2],ed:L[f+16>>2],Pc:L[f+20>>2],yd:L[f+24>>2],zd:L[f+28>>2],pf:L[f+32>>2],Re:L[f+
    36>>2],Ue:n?J(n):""};d=J(d);n={"%c":"%a %b %d %H:%M:%S %Y","%D":"%m/%d/%y","%F":"%Y-%m-%d","%h":"%b","%r":"%I:%M:%S %p","%R":"%H:%M","%T":"%H:%M:%S","%x":"%m/%d/%y","%X":"%H:%M:%S","%Ec":"%c","%EC":"%C","%Ex":"%m/%d/%y","%EX":"%H:%M:%S","%Ey":"%y","%EY":"%Y","%Od":"%d","%Oe":"%e","%OH":"%H","%OI":"%I","%Om":"%m","%OM":"%M","%OS":"%S","%Ou":"%u","%OU":"%U","%OV":"%V","%Ow":"%w","%OW":"%W","%Oy":"%y"};for(var q in n)d=d.replace(new RegExp(q,"g"),n[q]);var w="Sunday Monday Tuesday Wednesday Thursday Friday Saturday".split(" "),
    y="January February March April May June July August September October November December".split(" ");n={"%a":function(k){return w[k.yd].substring(0,3)},"%A":function(k){return w[k.yd]},"%b":function(k){return y[k.ed].substring(0,3)},"%B":function(k){return y[k.ed]},"%C":function(k){return h((k.Pc+1900)/100|0,2)},"%d":function(k){return h(k.qd,2)},"%e":function(k){return g(k.qd,2," ")},"%g":function(k){return t(k).toString().substring(2)},"%G":function(k){return t(k)},"%H":function(k){return h(k.xd,
    2)},"%I":function(k){k=k.xd;0==k?k=12:12<k&&(k-=12);return h(k,2)},"%j":function(k){return h(k.qd+sb(rb(k.Pc+1900)?tb:ub,k.ed-1),3)},"%m":function(k){return h(k.ed+1,2)},"%M":function(k){return h(k.Se,2)},"%n":function(){return "\n"},"%p":function(k){return 0<=k.xd&&12>k.xd?"AM":"PM"},"%S":function(k){return h(k.Te,2)},"%t":function(){return "\t"},"%u":function(k){return k.yd||7},"%U":function(k){var r=new Date(k.Pc+1900,0,1),v=0===r.getDay()?r:vb(r,7-r.getDay());k=new Date(k.Pc+1900,k.ed,k.qd);return 0>
    l(v,k)?h(Math.ceil((31-v.getDate()+(sb(rb(k.getFullYear())?tb:ub,k.getMonth()-1)-31)+k.getDate())/7),2):0===l(v,r)?"01":"00"},"%V":function(k){var r=new Date(k.Pc+1901,0,4),v=p(new Date(k.Pc+1900,0,4));r=p(r);var B=vb(new Date(k.Pc+1900,0,1),k.zd);return 0>l(B,v)?"53":0>=l(r,B)?"01":h(Math.ceil((v.getFullYear()<k.Pc+1900?k.zd+32-v.getDate():k.zd+1-v.getDate())/7),2)},"%w":function(k){return k.yd},"%W":function(k){var r=new Date(k.Pc,0,1),v=1===r.getDay()?r:vb(r,0===r.getDay()?1:7-r.getDay()+1);k=
    new Date(k.Pc+1900,k.ed,k.qd);return 0>l(v,k)?h(Math.ceil((31-v.getDate()+(sb(rb(k.getFullYear())?tb:ub,k.getMonth()-1)-31)+k.getDate())/7),2):0===l(v,r)?"01":"00"},"%y":function(k){return (k.Pc+1900).toString().substring(2)},"%Y":function(k){return k.Pc+1900},"%z":function(k){k=k.Re;var r=0<=k;k=Math.abs(k)/60;return (r?"+":"-")+String("0000"+(k/60*100+k%60)).slice(-4)},"%Z":function(k){return k.Ue},"%%":function(){return "%"}};for(q in n)d.includes(q)&&(d=d.replace(new RegExp(q,"g"),n[q](f)));q=hb(d,
    !1);if(q.length>b)return 0;H.set(q,a);return q.length-1},k:function(a){if(m){if(!a)return 1;a=J(a);if(!a.length)return 0;a=require("child_process").nf(a,[],{mf:!0,stdio:"inherit"});var b=(d,f)=>d<<8|f;return null===a.status?b(0,(d=>{switch(d){case "SIGHUP":return 1;case "SIGQUIT":return 3;case "SIGFPE":return 8;case "SIGKILL":return 9;case "SIGALRM":return 14;case "SIGTERM":return 15}return 2})(a.signal)):a.status<<8|0}if(!a)return 0;L[xb()>>2]=52;return -1},b:function(a){var b=Date.now()/1E3|0;a&&
    (L[a>>2]=b);return b}};
    (function(){function a(g){c.asm=g.exports;qa=c.asm.F;Fa();M=c.asm.db;Ha.unshift(c.asm.G);Na();}function b(g){a(g.instance);}function d(g){return Ra().then(function(h){return WebAssembly.instantiate(h,f)}).then(function(h){return h}).then(g,function(h){A("failed to asynchronously prepare wasm: "+h);F(h);})}var f={a:zb};Ma();if(c.instantiateWasm)try{return c.instantiateWasm(f,a)}catch(g){return A("Module.instantiateWasm callback failed with error: "+g),!1}(function(){return E||"function"!==typeof WebAssembly.instantiateStreaming||
    Oa()||O.startsWith("file://")||"function"!==typeof fetch?d(b):fetch(O,{credentials:"same-origin"}).then(function(g){return WebAssembly.instantiateStreaming(g,f).then(b,function(h){A("wasm streaming compile failed: "+h);A("falling back to ArrayBuffer instantiation");return d(b)})})})().catch(ca);return {}})();c.___wasm_call_ctors=function(){return (c.___wasm_call_ctors=c.asm.G).apply(null,arguments)};c._lua_checkstack=function(){return (c._lua_checkstack=c.asm.H).apply(null,arguments)};
    c._lua_xmove=function(){return (c._lua_xmove=c.asm.I).apply(null,arguments)};c._lua_atpanic=function(){return (c._lua_atpanic=c.asm.J).apply(null,arguments)};c._lua_version=function(){return (c._lua_version=c.asm.K).apply(null,arguments)};c._lua_absindex=function(){return (c._lua_absindex=c.asm.L).apply(null,arguments)};c._lua_gettop=function(){return (c._lua_gettop=c.asm.M).apply(null,arguments)};c._lua_settop=function(){return (c._lua_settop=c.asm.N).apply(null,arguments)};
    c._lua_closeslot=function(){return (c._lua_closeslot=c.asm.O).apply(null,arguments)};c._lua_rotate=function(){return (c._lua_rotate=c.asm.P).apply(null,arguments)};c._lua_copy=function(){return (c._lua_copy=c.asm.Q).apply(null,arguments)};c._lua_pushvalue=function(){return (c._lua_pushvalue=c.asm.R).apply(null,arguments)};c._lua_type=function(){return (c._lua_type=c.asm.S).apply(null,arguments)};c._lua_typename=function(){return (c._lua_typename=c.asm.T).apply(null,arguments)};
    c._lua_iscfunction=function(){return (c._lua_iscfunction=c.asm.U).apply(null,arguments)};c._lua_isinteger=function(){return (c._lua_isinteger=c.asm.V).apply(null,arguments)};c._lua_isnumber=function(){return (c._lua_isnumber=c.asm.W).apply(null,arguments)};c._lua_isstring=function(){return (c._lua_isstring=c.asm.X).apply(null,arguments)};c._lua_isuserdata=function(){return (c._lua_isuserdata=c.asm.Y).apply(null,arguments)};c._lua_rawequal=function(){return (c._lua_rawequal=c.asm.Z).apply(null,arguments)};
    c._lua_arith=function(){return (c._lua_arith=c.asm._).apply(null,arguments)};c._lua_compare=function(){return (c._lua_compare=c.asm.$).apply(null,arguments)};c._lua_stringtonumber=function(){return (c._lua_stringtonumber=c.asm.aa).apply(null,arguments)};c._lua_tonumberx=function(){return (c._lua_tonumberx=c.asm.ba).apply(null,arguments)};c._lua_tointegerx=function(){return (c._lua_tointegerx=c.asm.ca).apply(null,arguments)};c._lua_toboolean=function(){return (c._lua_toboolean=c.asm.da).apply(null,arguments)};
    c._lua_tolstring=function(){return (c._lua_tolstring=c.asm.ea).apply(null,arguments)};c._lua_rawlen=function(){return (c._lua_rawlen=c.asm.fa).apply(null,arguments)};c._lua_tocfunction=function(){return (c._lua_tocfunction=c.asm.ga).apply(null,arguments)};c._lua_touserdata=function(){return (c._lua_touserdata=c.asm.ha).apply(null,arguments)};c._lua_tothread=function(){return (c._lua_tothread=c.asm.ia).apply(null,arguments)};c._lua_topointer=function(){return (c._lua_topointer=c.asm.ja).apply(null,arguments)};
    c._lua_pushnil=function(){return (c._lua_pushnil=c.asm.ka).apply(null,arguments)};c._lua_pushnumber=function(){return (c._lua_pushnumber=c.asm.la).apply(null,arguments)};c._lua_pushinteger=function(){return (c._lua_pushinteger=c.asm.ma).apply(null,arguments)};c._lua_pushlstring=function(){return (c._lua_pushlstring=c.asm.na).apply(null,arguments)};c._lua_pushstring=function(){return (c._lua_pushstring=c.asm.oa).apply(null,arguments)};
    c._lua_pushcclosure=function(){return (c._lua_pushcclosure=c.asm.pa).apply(null,arguments)};c._lua_pushboolean=function(){return (c._lua_pushboolean=c.asm.qa).apply(null,arguments)};c._lua_pushlightuserdata=function(){return (c._lua_pushlightuserdata=c.asm.ra).apply(null,arguments)};c._lua_pushthread=function(){return (c._lua_pushthread=c.asm.sa).apply(null,arguments)};c._lua_getglobal=function(){return (c._lua_getglobal=c.asm.ta).apply(null,arguments)};
    c._lua_gettable=function(){return (c._lua_gettable=c.asm.ua).apply(null,arguments)};c._lua_getfield=function(){return (c._lua_getfield=c.asm.va).apply(null,arguments)};c._lua_geti=function(){return (c._lua_geti=c.asm.wa).apply(null,arguments)};c._lua_rawget=function(){return (c._lua_rawget=c.asm.xa).apply(null,arguments)};c._lua_rawgeti=function(){return (c._lua_rawgeti=c.asm.ya).apply(null,arguments)};c._lua_rawgetp=function(){return (c._lua_rawgetp=c.asm.za).apply(null,arguments)};
    c._lua_createtable=function(){return (c._lua_createtable=c.asm.Aa).apply(null,arguments)};c._lua_getmetatable=function(){return (c._lua_getmetatable=c.asm.Ba).apply(null,arguments)};c._lua_getiuservalue=function(){return (c._lua_getiuservalue=c.asm.Ca).apply(null,arguments)};c._lua_setglobal=function(){return (c._lua_setglobal=c.asm.Da).apply(null,arguments)};c._lua_settable=function(){return (c._lua_settable=c.asm.Ea).apply(null,arguments)};
    c._lua_setfield=function(){return (c._lua_setfield=c.asm.Fa).apply(null,arguments)};c._lua_seti=function(){return (c._lua_seti=c.asm.Ga).apply(null,arguments)};c._lua_rawset=function(){return (c._lua_rawset=c.asm.Ha).apply(null,arguments)};c._lua_rawsetp=function(){return (c._lua_rawsetp=c.asm.Ia).apply(null,arguments)};c._lua_rawseti=function(){return (c._lua_rawseti=c.asm.Ja).apply(null,arguments)};c._lua_setmetatable=function(){return (c._lua_setmetatable=c.asm.Ka).apply(null,arguments)};
    c._lua_setiuservalue=function(){return (c._lua_setiuservalue=c.asm.La).apply(null,arguments)};c._lua_callk=function(){return (c._lua_callk=c.asm.Ma).apply(null,arguments)};c._lua_pcallk=function(){return (c._lua_pcallk=c.asm.Na).apply(null,arguments)};c._lua_load=function(){return (c._lua_load=c.asm.Oa).apply(null,arguments)};c._lua_dump=function(){return (c._lua_dump=c.asm.Pa).apply(null,arguments)};c._lua_status=function(){return (c._lua_status=c.asm.Qa).apply(null,arguments)};
    c._lua_error=function(){return (c._lua_error=c.asm.Ra).apply(null,arguments)};c._lua_next=function(){return (c._lua_next=c.asm.Sa).apply(null,arguments)};c._lua_toclose=function(){return (c._lua_toclose=c.asm.Ta).apply(null,arguments)};c._lua_concat=function(){return (c._lua_concat=c.asm.Ua).apply(null,arguments)};c._lua_len=function(){return (c._lua_len=c.asm.Va).apply(null,arguments)};c._lua_getallocf=function(){return (c._lua_getallocf=c.asm.Wa).apply(null,arguments)};
    c._lua_setallocf=function(){return (c._lua_setallocf=c.asm.Xa).apply(null,arguments)};c._lua_setwarnf=function(){return (c._lua_setwarnf=c.asm.Ya).apply(null,arguments)};c._lua_warning=function(){return (c._lua_warning=c.asm.Za).apply(null,arguments)};c._lua_newuserdatauv=function(){return (c._lua_newuserdatauv=c.asm._a).apply(null,arguments)};c._lua_getupvalue=function(){return (c._lua_getupvalue=c.asm.$a).apply(null,arguments)};
    c._lua_setupvalue=function(){return (c._lua_setupvalue=c.asm.ab).apply(null,arguments)};c._lua_upvalueid=function(){return (c._lua_upvalueid=c.asm.bb).apply(null,arguments)};c._lua_upvaluejoin=function(){return (c._lua_upvaluejoin=c.asm.cb).apply(null,arguments)};c._luaL_traceback=function(){return (c._luaL_traceback=c.asm.eb).apply(null,arguments)};c._lua_getstack=function(){return (c._lua_getstack=c.asm.fb).apply(null,arguments)};c._lua_getinfo=function(){return (c._lua_getinfo=c.asm.gb).apply(null,arguments)};
    c._luaL_buffinit=function(){return (c._luaL_buffinit=c.asm.hb).apply(null,arguments)};c._luaL_addstring=function(){return (c._luaL_addstring=c.asm.ib).apply(null,arguments)};c._luaL_prepbuffsize=function(){return (c._luaL_prepbuffsize=c.asm.jb).apply(null,arguments)};c._luaL_addvalue=function(){return (c._luaL_addvalue=c.asm.kb).apply(null,arguments)};c._luaL_pushresult=function(){return (c._luaL_pushresult=c.asm.lb).apply(null,arguments)};
    c._luaL_argerror=function(){return (c._luaL_argerror=c.asm.mb).apply(null,arguments)};c._luaL_typeerror=function(){return (c._luaL_typeerror=c.asm.nb).apply(null,arguments)};c._luaL_getmetafield=function(){return (c._luaL_getmetafield=c.asm.ob).apply(null,arguments)};c._luaL_where=function(){return (c._luaL_where=c.asm.pb).apply(null,arguments)};c._luaL_fileresult=function(){return (c._luaL_fileresult=c.asm.qb).apply(null,arguments)};
    var xb=c.___errno_location=function(){return (xb=c.___errno_location=c.asm.rb).apply(null,arguments)};c._luaL_execresult=function(){return (c._luaL_execresult=c.asm.sb).apply(null,arguments)};c._luaL_newmetatable=function(){return (c._luaL_newmetatable=c.asm.tb).apply(null,arguments)};c._luaL_setmetatable=function(){return (c._luaL_setmetatable=c.asm.ub).apply(null,arguments)};c._luaL_testudata=function(){return (c._luaL_testudata=c.asm.vb).apply(null,arguments)};
    c._luaL_checkudata=function(){return (c._luaL_checkudata=c.asm.wb).apply(null,arguments)};c._luaL_optlstring=function(){return (c._luaL_optlstring=c.asm.xb).apply(null,arguments)};c._luaL_checklstring=function(){return (c._luaL_checklstring=c.asm.yb).apply(null,arguments)};c._luaL_checkstack=function(){return (c._luaL_checkstack=c.asm.zb).apply(null,arguments)};c._luaL_checktype=function(){return (c._luaL_checktype=c.asm.Ab).apply(null,arguments)};
    c._luaL_checkany=function(){return (c._luaL_checkany=c.asm.Bb).apply(null,arguments)};c._luaL_checknumber=function(){return (c._luaL_checknumber=c.asm.Cb).apply(null,arguments)};c._luaL_optnumber=function(){return (c._luaL_optnumber=c.asm.Db).apply(null,arguments)};c._luaL_checkinteger=function(){return (c._luaL_checkinteger=c.asm.Eb).apply(null,arguments)};c._luaL_optinteger=function(){return (c._luaL_optinteger=c.asm.Fb).apply(null,arguments)};
    c._luaL_setfuncs=function(){return (c._luaL_setfuncs=c.asm.Gb).apply(null,arguments)};c._luaL_addlstring=function(){return (c._luaL_addlstring=c.asm.Hb).apply(null,arguments)};c._luaL_pushresultsize=function(){return (c._luaL_pushresultsize=c.asm.Ib).apply(null,arguments)};c._luaL_buffinitsize=function(){return (c._luaL_buffinitsize=c.asm.Jb).apply(null,arguments)};c._luaL_ref=function(){return (c._luaL_ref=c.asm.Kb).apply(null,arguments)};
    c._luaL_unref=function(){return (c._luaL_unref=c.asm.Lb).apply(null,arguments)};c._luaL_loadfilex=function(){return (c._luaL_loadfilex=c.asm.Mb).apply(null,arguments)};c._luaL_loadbufferx=function(){return (c._luaL_loadbufferx=c.asm.Nb).apply(null,arguments)};c._luaL_loadstring=function(){return (c._luaL_loadstring=c.asm.Ob).apply(null,arguments)};c._luaL_callmeta=function(){return (c._luaL_callmeta=c.asm.Pb).apply(null,arguments)};c._luaL_len=function(){return (c._luaL_len=c.asm.Qb).apply(null,arguments)};
    c._luaL_tolstring=function(){return (c._luaL_tolstring=c.asm.Rb).apply(null,arguments)};c._luaL_getsubtable=function(){return (c._luaL_getsubtable=c.asm.Sb).apply(null,arguments)};c._luaL_requiref=function(){return (c._luaL_requiref=c.asm.Tb).apply(null,arguments)};c._luaL_addgsub=function(){return (c._luaL_addgsub=c.asm.Ub).apply(null,arguments)};c._luaL_gsub=function(){return (c._luaL_gsub=c.asm.Vb).apply(null,arguments)};c._luaL_newstate=function(){return (c._luaL_newstate=c.asm.Wb).apply(null,arguments)};
    c._lua_newstate=function(){return (c._lua_newstate=c.asm.Xb).apply(null,arguments)};c._free=function(){return (c._free=c.asm.Yb).apply(null,arguments)};c._realloc=function(){return (c._realloc=c.asm.Zb).apply(null,arguments)};c._luaL_checkversion_=function(){return (c._luaL_checkversion_=c.asm._b).apply(null,arguments)};c._luaopen_base=function(){return (c._luaopen_base=c.asm.$b).apply(null,arguments)};c._luaopen_coroutine=function(){return (c._luaopen_coroutine=c.asm.ac).apply(null,arguments)};
    c._lua_newthread=function(){return (c._lua_newthread=c.asm.bc).apply(null,arguments)};c._lua_yieldk=function(){return (c._lua_yieldk=c.asm.cc).apply(null,arguments)};c._lua_isyieldable=function(){return (c._lua_isyieldable=c.asm.dc).apply(null,arguments)};c._lua_resetthread=function(){return (c._lua_resetthread=c.asm.ec).apply(null,arguments)};c._lua_resume=function(){return (c._lua_resume=c.asm.fc).apply(null,arguments)};c._luaopen_debug=function(){return (c._luaopen_debug=c.asm.gc).apply(null,arguments)};
    c._lua_gethookmask=function(){return (c._lua_gethookmask=c.asm.hc).apply(null,arguments)};c._lua_gethook=function(){return (c._lua_gethook=c.asm.ic).apply(null,arguments)};c._lua_gethookcount=function(){return (c._lua_gethookcount=c.asm.jc).apply(null,arguments)};c._lua_getlocal=function(){return (c._lua_getlocal=c.asm.kc).apply(null,arguments)};c._lua_sethook=function(){return (c._lua_sethook=c.asm.lc).apply(null,arguments)};c._lua_setlocal=function(){return (c._lua_setlocal=c.asm.mc).apply(null,arguments)};
    c._lua_setcstacklimit=function(){return (c._lua_setcstacklimit=c.asm.nc).apply(null,arguments)};var Aa=c._malloc=function(){return (Aa=c._malloc=c.asm.oc).apply(null,arguments)};c._luaL_openlibs=function(){return (c._luaL_openlibs=c.asm.pc).apply(null,arguments)};c._luaopen_package=function(){return (c._luaopen_package=c.asm.qc).apply(null,arguments)};c._luaopen_table=function(){return (c._luaopen_table=c.asm.rc).apply(null,arguments)};
    c._luaopen_io=function(){return (c._luaopen_io=c.asm.sc).apply(null,arguments)};c._luaopen_os=function(){return (c._luaopen_os=c.asm.tc).apply(null,arguments)};c._luaopen_string=function(){return (c._luaopen_string=c.asm.uc).apply(null,arguments)};c._luaopen_math=function(){return (c._luaopen_math=c.asm.vc).apply(null,arguments)};c._luaopen_utf8=function(){return (c._luaopen_utf8=c.asm.wc).apply(null,arguments)};c._lua_close=function(){return (c._lua_close=c.asm.xc).apply(null,arguments)};
    var S=c.__get_tzname=function(){return (S=c.__get_tzname=c.asm.yc).apply(null,arguments)},Ya=c.__get_daylight=function(){return (Ya=c.__get_daylight=c.asm.zc).apply(null,arguments)},Xa=c.__get_timezone=function(){return (Xa=c.__get_timezone=c.asm.Ac).apply(null,arguments)},Bb=c._setThrew=function(){return (Bb=c._setThrew=c.asm.Bc).apply(null,arguments)},va=c.stackSave=function(){return (va=c.stackSave=c.asm.Cc).apply(null,arguments)},wa=c.stackRestore=function(){return (wa=c.stackRestore=c.asm.Dc).apply(null,
    arguments)},ta=c.stackAlloc=function(){return (ta=c.stackAlloc=c.asm.Ec).apply(null,arguments)};function yb(a,b,d){var f=va();try{Ta(a)(b,d);}catch(g){wa(f);if(g!==g+0&&"longjmp"!==g)throw g;Bb(1,0);}}c.cwrap=function(a,b,d,f){d=d||[];var g=d.every(function(h){return "number"===h});return "string"!==b&&g&&!f?c["_"+a]:function(){return sa(a,b,d,arguments)}};
    c.setValue=function(a,b,d){d=d||"i8";"*"===d.charAt(d.length-1)&&(d="i32");switch(d){case "i1":H[a>>0]=b;break;case "i8":H[a>>0]=b;break;case "i16":Ca[a>>1]=b;break;case "i32":L[a>>2]=b;break;case "i64":Q=[b>>>0,(P=b,1<=+Math.abs(P)?0<P?(Math.min(+Math.floor(P/4294967296),4294967295)|0)>>>0:~~+Math.ceil((P-+(~~P>>>0))/4294967296)>>>0:0)];L[a>>2]=Q[0];L[a+4>>2]=Q[1];break;case "float":Da[a>>2]=b;break;case "double":Ea[a>>3]=b;break;default:F("invalid type for setValue: "+d);}};
    c.getValue=function(a,b){b=b||"i8";"*"===b.charAt(b.length-1)&&(b="i32");switch(b){case "i1":return H[a>>0];case "i8":return H[a>>0];case "i16":return Ca[a>>1];case "i32":return L[a>>2];case "i64":return L[a>>2];case "float":return Da[a>>2];case "double":return Number(Ea[a>>3]);default:F("invalid type for getValue: "+b);}return null};
    c.addFunction=function(a,b){if(!D){D=new WeakMap;for(var d=M.length,f=0;f<0+d;f++){var g=Ta(f);g&&D.set(g,f);}}if(D.has(a))return D.get(a);if(oa.length)d=oa.pop();else {try{M.grow(1);}catch(p){if(!(p instanceof RangeError))throw p;throw "Unable to grow wasm table. Set ALLOW_TABLE_GROWTH.";}d=M.length-1;}try{f=d,M.set(f,a),R[f]=a;}catch(p){if(!(p instanceof TypeError))throw p;if("function"===typeof WebAssembly.Function){g={i:"i32",j:"i64",f:"f32",d:"f64"};var h={parameters:[],results:"v"==b[0]?[]:[g[b[0]]]};
    for(f=1;f<b.length;++f)h.parameters.push(g[b[f]]);f=new WebAssembly.Function(h,a);}else {g=[1,0,1,96];h=b.slice(0,1);b=b.slice(1);var l={i:127,j:126,f:125,d:124};g.push(b.length);for(f=0;f<b.length;++f)g.push(l[b[f]]);"v"==h?g.push(0):g=g.concat([1,l[h]]);g[1]=g.length-2;b=new Uint8Array([0,97,115,109,1,0,0,0].concat(g,[2,7,1,1,101,1,102,0,0,7,5,1,1,102,0,0]));b=new WebAssembly.Module(b);f=(new WebAssembly.Instance(b,{e:{f:a}})).exports.f;}b=d;M.set(b,f);R[b]=f;}D.set(a,d);return d};
    c.removeFunction=function(a){D.delete(Ta(a));oa.push(a);};c.ENV=ob;c.FS=W;var Cb;function ma(a){this.name="ExitStatus";this.message="Program terminated with exit("+a+")";this.status=a;}La=function Db(){Cb||Eb();Cb||(La=Db);};
    function Eb(){function a(){if(!Cb&&(Cb=!0,c.calledRun=!0,!ra)){c.noFSInit||W.jd.Jd||W.jd();W.ie=!1;Sa(Ha);ba(c);if(c.onRuntimeInitialized)c.onRuntimeInitialized();if(c.postRun)for("function"==typeof c.postRun&&(c.postRun=[c.postRun]);c.postRun.length;){var b=c.postRun.shift();Ia.unshift(b);}Sa(Ia);}}if(!(0<N)){if(c.preRun)for("function"==typeof c.preRun&&(c.preRun=[c.preRun]);c.preRun.length;)Ja();Sa(Ga);0<N||(c.setStatus?(c.setStatus("Running..."),setTimeout(function(){setTimeout(function(){c.setStatus("");},
    1);a();},1)):a());}}c.run=Eb;if(c.preInit)for("function"==typeof c.preInit&&(c.preInit=[c.preInit]);0<c.preInit.length;)c.preInit.pop()();Eb();


      return initWasmModule.ready
    }
    );
    })();

    class LuaWasm {
        constructor(module) {
            this.referenceTracker = new WeakMap();
            this.referenceMap = new Map();
            this.availableReferences = [];
            this.module = module;
            this.luaL_checkversion_ = this.module.cwrap('luaL_checkversion_', null, ['number', 'number', 'number']);
            this.luaL_getmetafield = this.module.cwrap('luaL_getmetafield', 'number', ['number', 'number', 'string']);
            this.luaL_callmeta = this.module.cwrap('luaL_callmeta', 'number', ['number', 'number', 'string']);
            this.luaL_tolstring = this.module.cwrap('luaL_tolstring', 'string', ['number', 'number', 'number']);
            this.luaL_argerror = this.module.cwrap('luaL_argerror', 'number', ['number', 'number', 'string']);
            this.luaL_typeerror = this.module.cwrap('luaL_typeerror', 'number', ['number', 'number', 'string']);
            this.luaL_checklstring = this.module.cwrap('luaL_checklstring', 'string', ['number', 'number', 'number']);
            this.luaL_optlstring = this.module.cwrap('luaL_optlstring', 'string', ['number', 'number', 'string', 'number']);
            this.luaL_checknumber = this.module.cwrap('luaL_checknumber', 'number', ['number', 'number']);
            this.luaL_optnumber = this.module.cwrap('luaL_optnumber', 'number', ['number', 'number', 'number']);
            this.luaL_checkinteger = this.module.cwrap('luaL_checkinteger', 'number', ['number', 'number']);
            this.luaL_optinteger = this.module.cwrap('luaL_optinteger', 'number', ['number', 'number', 'number']);
            this.luaL_checkstack = this.module.cwrap('luaL_checkstack', null, ['number', 'number', 'string']);
            this.luaL_checktype = this.module.cwrap('luaL_checktype', null, ['number', 'number', 'number']);
            this.luaL_checkany = this.module.cwrap('luaL_checkany', null, ['number', 'number']);
            this.luaL_newmetatable = this.module.cwrap('luaL_newmetatable', 'number', ['number', 'string']);
            this.luaL_setmetatable = this.module.cwrap('luaL_setmetatable', null, ['number', 'string']);
            this.luaL_testudata = this.module.cwrap('luaL_testudata', 'number', ['number', 'number', 'string']);
            this.luaL_checkudata = this.module.cwrap('luaL_checkudata', 'number', ['number', 'number', 'string']);
            this.luaL_where = this.module.cwrap('luaL_where', null, ['number', 'number']);
            this.luaL_fileresult = this.module.cwrap('luaL_fileresult', 'number', ['number', 'number', 'string']);
            this.luaL_execresult = this.module.cwrap('luaL_execresult', 'number', ['number', 'number']);
            this.luaL_ref = this.module.cwrap('luaL_ref', 'number', ['number', 'number']);
            this.luaL_unref = this.module.cwrap('luaL_unref', null, ['number', 'number', 'number']);
            this.luaL_loadfilex = this.module.cwrap('luaL_loadfilex', 'number', ['number', 'string', 'string']);
            this.luaL_loadbufferx = this.module.cwrap('luaL_loadbufferx', 'number', ['number', 'string', 'number', 'string', 'string']);
            this.luaL_loadstring = this.module.cwrap('luaL_loadstring', 'number', ['number', 'string']);
            this.luaL_newstate = this.module.cwrap('luaL_newstate', 'number', []);
            this.luaL_len = this.module.cwrap('luaL_len', 'number', ['number', 'number']);
            this.luaL_addgsub = this.module.cwrap('luaL_addgsub', null, ['number', 'string', 'string', 'string']);
            this.luaL_gsub = this.module.cwrap('luaL_gsub', 'string', ['number', 'string', 'string', 'string']);
            this.luaL_setfuncs = this.module.cwrap('luaL_setfuncs', null, ['number', 'number', 'number']);
            this.luaL_getsubtable = this.module.cwrap('luaL_getsubtable', 'number', ['number', 'number', 'string']);
            this.luaL_traceback = this.module.cwrap('luaL_traceback', null, ['number', 'number', 'string', 'number']);
            this.luaL_requiref = this.module.cwrap('luaL_requiref', null, ['number', 'string', 'number', 'number']);
            this.luaL_buffinit = this.module.cwrap('luaL_buffinit', null, ['number', 'number']);
            this.luaL_prepbuffsize = this.module.cwrap('luaL_prepbuffsize', 'string', ['number', 'number']);
            this.luaL_addlstring = this.module.cwrap('luaL_addlstring', null, ['number', 'string', 'number']);
            this.luaL_addstring = this.module.cwrap('luaL_addstring', null, ['number', 'string']);
            this.luaL_addvalue = this.module.cwrap('luaL_addvalue', null, ['number']);
            this.luaL_pushresult = this.module.cwrap('luaL_pushresult', null, ['number']);
            this.luaL_pushresultsize = this.module.cwrap('luaL_pushresultsize', null, ['number', 'number']);
            this.luaL_buffinitsize = this.module.cwrap('luaL_buffinitsize', 'string', ['number', 'number', 'number']);
            this.lua_newstate = this.module.cwrap('lua_newstate', 'number', ['number', 'number']);
            this.lua_close = this.module.cwrap('lua_close', null, ['number']);
            this.lua_newthread = this.module.cwrap('lua_newthread', 'number', ['number']);
            this.lua_resetthread = this.module.cwrap('lua_resetthread', 'number', ['number']);
            this.lua_atpanic = this.module.cwrap('lua_atpanic', 'number', ['number', 'number']);
            this.lua_version = this.module.cwrap('lua_version', 'number', ['number']);
            this.lua_absindex = this.module.cwrap('lua_absindex', 'number', ['number', 'number']);
            this.lua_gettop = this.module.cwrap('lua_gettop', 'number', ['number']);
            this.lua_settop = this.module.cwrap('lua_settop', null, ['number', 'number']);
            this.lua_pushvalue = this.module.cwrap('lua_pushvalue', null, ['number', 'number']);
            this.lua_rotate = this.module.cwrap('lua_rotate', null, ['number', 'number', 'number']);
            this.lua_copy = this.module.cwrap('lua_copy', null, ['number', 'number', 'number']);
            this.lua_checkstack = this.module.cwrap('lua_checkstack', 'number', ['number', 'number']);
            this.lua_xmove = this.module.cwrap('lua_xmove', null, ['number', 'number', 'number']);
            this.lua_isnumber = this.module.cwrap('lua_isnumber', 'number', ['number', 'number']);
            this.lua_isstring = this.module.cwrap('lua_isstring', 'number', ['number', 'number']);
            this.lua_iscfunction = this.module.cwrap('lua_iscfunction', 'number', ['number', 'number']);
            this.lua_isinteger = this.module.cwrap('lua_isinteger', 'number', ['number', 'number']);
            this.lua_isuserdata = this.module.cwrap('lua_isuserdata', 'number', ['number', 'number']);
            this.lua_type = this.module.cwrap('lua_type', 'number', ['number', 'number']);
            this.lua_typename = this.module.cwrap('lua_typename', 'string', ['number', 'number']);
            this.lua_tonumberx = this.module.cwrap('lua_tonumberx', 'number', ['number', 'number', 'number']);
            this.lua_tointegerx = this.module.cwrap('lua_tointegerx', 'number', ['number', 'number', 'number']);
            this.lua_toboolean = this.module.cwrap('lua_toboolean', 'number', ['number', 'number']);
            this.lua_tolstring = this.module.cwrap('lua_tolstring', 'string', ['number', 'number', 'number']);
            this.lua_rawlen = this.module.cwrap('lua_rawlen', 'number', ['number', 'number']);
            this.lua_tocfunction = this.module.cwrap('lua_tocfunction', 'number', ['number', 'number']);
            this.lua_touserdata = this.module.cwrap('lua_touserdata', 'number', ['number', 'number']);
            this.lua_tothread = this.module.cwrap('lua_tothread', 'number', ['number', 'number']);
            this.lua_topointer = this.module.cwrap('lua_topointer', 'number', ['number', 'number']);
            this.lua_arith = this.module.cwrap('lua_arith', null, ['number', 'number']);
            this.lua_rawequal = this.module.cwrap('lua_rawequal', 'number', ['number', 'number', 'number']);
            this.lua_compare = this.module.cwrap('lua_compare', 'number', ['number', 'number', 'number', 'number']);
            this.lua_pushnil = this.module.cwrap('lua_pushnil', null, ['number']);
            this.lua_pushnumber = this.module.cwrap('lua_pushnumber', null, ['number', 'number']);
            this.lua_pushinteger = this.module.cwrap('lua_pushinteger', null, ['number', 'number']);
            this.lua_pushlstring = this.module.cwrap('lua_pushlstring', 'string', ['number', 'string', 'number']);
            this.lua_pushstring = this.module.cwrap('lua_pushstring', 'string', ['number', 'string']);
            this.lua_pushcclosure = this.module.cwrap('lua_pushcclosure', null, ['number', 'number', 'number']);
            this.lua_pushboolean = this.module.cwrap('lua_pushboolean', null, ['number', 'number']);
            this.lua_pushlightuserdata = this.module.cwrap('lua_pushlightuserdata', null, ['number', 'number']);
            this.lua_pushthread = this.module.cwrap('lua_pushthread', 'number', ['number']);
            this.lua_getglobal = this.module.cwrap('lua_getglobal', 'number', ['number', 'string']);
            this.lua_gettable = this.module.cwrap('lua_gettable', 'number', ['number', 'number']);
            this.lua_getfield = this.module.cwrap('lua_getfield', 'number', ['number', 'number', 'string']);
            this.lua_geti = this.module.cwrap('lua_geti', 'number', ['number', 'number', 'number']);
            this.lua_rawget = this.module.cwrap('lua_rawget', 'number', ['number', 'number']);
            this.lua_rawgeti = this.module.cwrap('lua_rawgeti', 'number', ['number', 'number', 'number']);
            this.lua_rawgetp = this.module.cwrap('lua_rawgetp', 'number', ['number', 'number', 'number']);
            this.lua_createtable = this.module.cwrap('lua_createtable', null, ['number', 'number', 'number']);
            this.lua_newuserdatauv = this.module.cwrap('lua_newuserdatauv', 'number', ['number', 'number', 'number']);
            this.lua_getmetatable = this.module.cwrap('lua_getmetatable', 'number', ['number', 'number']);
            this.lua_getiuservalue = this.module.cwrap('lua_getiuservalue', 'number', ['number', 'number', 'number']);
            this.lua_setglobal = this.module.cwrap('lua_setglobal', null, ['number', 'string']);
            this.lua_settable = this.module.cwrap('lua_settable', null, ['number', 'number']);
            this.lua_setfield = this.module.cwrap('lua_setfield', null, ['number', 'number', 'string']);
            this.lua_seti = this.module.cwrap('lua_seti', null, ['number', 'number', 'number']);
            this.lua_rawset = this.module.cwrap('lua_rawset', null, ['number', 'number']);
            this.lua_rawseti = this.module.cwrap('lua_rawseti', null, ['number', 'number', 'number']);
            this.lua_rawsetp = this.module.cwrap('lua_rawsetp', null, ['number', 'number', 'number']);
            this.lua_setmetatable = this.module.cwrap('lua_setmetatable', 'number', ['number', 'number']);
            this.lua_setiuservalue = this.module.cwrap('lua_setiuservalue', 'number', ['number', 'number', 'number']);
            this.lua_callk = this.module.cwrap('lua_callk', null, ['number', 'number', 'number', 'number', 'number']);
            this.lua_pcallk = this.module.cwrap('lua_pcallk', 'number', ['number', 'number', 'number', 'number', 'number', 'number']);
            this.lua_load = this.module.cwrap('lua_load', 'number', ['number', 'number', 'number', 'string', 'string']);
            this.lua_dump = this.module.cwrap('lua_dump', 'number', ['number', 'number', 'number', 'number']);
            this.lua_yieldk = this.module.cwrap('lua_yieldk', 'number', ['number', 'number', 'number', 'number']);
            this.lua_resume = this.module.cwrap('lua_resume', 'number', ['number', 'number', 'number', 'number']);
            this.lua_status = this.module.cwrap('lua_status', 'number', ['number']);
            this.lua_isyieldable = this.module.cwrap('lua_isyieldable', 'number', ['number']);
            this.lua_setwarnf = this.module.cwrap('lua_setwarnf', null, ['number', 'number', 'number']);
            this.lua_warning = this.module.cwrap('lua_warning', null, ['number', 'string', 'number']);
            this.lua_error = this.module.cwrap('lua_error', 'number', ['number']);
            this.lua_next = this.module.cwrap('lua_next', 'number', ['number', 'number']);
            this.lua_concat = this.module.cwrap('lua_concat', null, ['number', 'number']);
            this.lua_len = this.module.cwrap('lua_len', null, ['number', 'number']);
            this.lua_stringtonumber = this.module.cwrap('lua_stringtonumber', 'number', ['number', 'string']);
            this.lua_getallocf = this.module.cwrap('lua_getallocf', 'number', ['number', 'number']);
            this.lua_setallocf = this.module.cwrap('lua_setallocf', null, ['number', 'number', 'number']);
            this.lua_toclose = this.module.cwrap('lua_toclose', null, ['number', 'number']);
            this.lua_closeslot = this.module.cwrap('lua_closeslot', null, ['number', 'number']);
            this.lua_getstack = this.module.cwrap('lua_getstack', 'number', ['number', 'number', 'number']);
            this.lua_getinfo = this.module.cwrap('lua_getinfo', 'number', ['number', 'string', 'number']);
            this.lua_getlocal = this.module.cwrap('lua_getlocal', 'string', ['number', 'number', 'number']);
            this.lua_setlocal = this.module.cwrap('lua_setlocal', 'string', ['number', 'number', 'number']);
            this.lua_getupvalue = this.module.cwrap('lua_getupvalue', 'string', ['number', 'number', 'number']);
            this.lua_setupvalue = this.module.cwrap('lua_setupvalue', 'string', ['number', 'number', 'number']);
            this.lua_upvalueid = this.module.cwrap('lua_upvalueid', 'number', ['number', 'number', 'number']);
            this.lua_upvaluejoin = this.module.cwrap('lua_upvaluejoin', null, ['number', 'number', 'number', 'number', 'number']);
            this.lua_sethook = this.module.cwrap('lua_sethook', null, ['number', 'number', 'number', 'number']);
            this.lua_gethook = this.module.cwrap('lua_gethook', 'number', ['number']);
            this.lua_gethookmask = this.module.cwrap('lua_gethookmask', 'number', ['number']);
            this.lua_gethookcount = this.module.cwrap('lua_gethookcount', 'number', ['number']);
            this.lua_setcstacklimit = this.module.cwrap('lua_setcstacklimit', 'number', ['number', 'number']);
            this.luaopen_base = this.module.cwrap('luaopen_base', 'number', ['number']);
            this.luaopen_coroutine = this.module.cwrap('luaopen_coroutine', 'number', ['number']);
            this.luaopen_table = this.module.cwrap('luaopen_table', 'number', ['number']);
            this.luaopen_io = this.module.cwrap('luaopen_io', 'number', ['number']);
            this.luaopen_os = this.module.cwrap('luaopen_os', 'number', ['number']);
            this.luaopen_string = this.module.cwrap('luaopen_string', 'number', ['number']);
            this.luaopen_utf8 = this.module.cwrap('luaopen_utf8', 'number', ['number']);
            this.luaopen_math = this.module.cwrap('luaopen_math', 'number', ['number']);
            this.luaopen_debug = this.module.cwrap('luaopen_debug', 'number', ['number']);
            this.luaopen_package = this.module.cwrap('luaopen_package', 'number', ['number']);
            this.luaL_openlibs = this.module.cwrap('luaL_openlibs', null, ['number']);
        }
        static async initialize(customName, env) {
            const module = await initWasmModule({
                print: console.log,
                printErr: console.error,
                locateFile: (path, scriptDirectory) => {
                    return customName || scriptDirectory + path;
                },
                preRun: (initializedModule) => {
                    if (typeof env === 'object') {
                        Object.entries(env).forEach(([k, v]) => (initializedModule.ENV[k] = v));
                    }
                },
            });
            return new LuaWasm(module);
        }
        lua_remove(luaState, index) {
            this.lua_rotate(luaState, index, -1);
            this.lua_pop(luaState, 1);
        }
        lua_pop(luaState, count) {
            this.lua_settop(luaState, -count - 1);
        }
        luaL_getmetatable(luaState, name) {
            return this.lua_getfield(luaState, LUA_REGISTRYINDEX, name);
        }
        lua_yield(luaState, count) {
            return this.lua_yieldk(luaState, count, 0, null);
        }
        lua_upvalueindex(index) {
            return LUA_REGISTRYINDEX - index;
        }
        ref(data) {
            const existing = this.referenceTracker.get(data);
            if (existing) {
                existing.refCount++;
                return existing.index;
            }
            const availableIndex = this.availableReferences.pop();
            const index = availableIndex === undefined ? this.referenceMap.size + 1 : availableIndex;
            this.referenceMap.set(index, data);
            this.referenceTracker.set(data, {
                refCount: 1,
                index,
            });
            return index;
        }
        unref(index) {
            const ref = this.referenceMap.get(index);
            if (ref === undefined) {
                return;
            }
            const metadata = this.referenceTracker.get(ref);
            if (metadata === undefined) {
                this.referenceTracker.delete(ref);
                this.availableReferences.push(index);
                return;
            }
            metadata.refCount--;
            if (metadata.refCount <= 0) {
                this.referenceTracker.delete(ref);
                this.referenceMap.delete(index);
                this.availableReferences.push(index);
            }
        }
        getRef(index) {
            return this.referenceMap.get(index);
        }
    }

    class LuaFactory {
        constructor(customWasmUri, environmentVariables) {
            this.customWasmUri = customWasmUri;
            this.environmentVariables = environmentVariables;
            if (this.customWasmUri === undefined) {
                const isBrowser = typeof window !== 'undefined' && typeof window.document !== 'undefined';
                if (isBrowser) {
                    this.customWasmUri = 'http://unpkg.com/wasmoon/dist/glue.wasm';
                }
            }
        }
        async mountFile(path, content) {
            await this.getLuaModule();
            this.mountFileSync(path, content);
        }
        mountFileSync(path, content) {
            if (!this.lua) {
                throw new Error("Module is not initialized, instead call 'mountFile' to ensure initialization");
            }
            const fileSep = path.lastIndexOf('/');
            const file = path.substring(fileSep + 1);
            const body = path.substring(0, path.length - file.length - 1);
            if (body.length > 0) {
                const parts = body.split('/').reverse();
                let parent = '';
                while (parts.length) {
                    const part = parts.pop();
                    if (!part) {
                        continue;
                    }
                    const current = `${parent}/${part}`;
                    try {
                        this.lua.module.FS.mkdir(current);
                    }
                    catch (err) {
                    }
                    parent = current;
                }
            }
            this.lua.module.FS.writeFile(path, content);
        }
        async createEngine(options) {
            return new LuaEngine(await this.getLuaModule(), options);
        }
        async getLuaModule() {
            if (!this.lua) {
                this.lua = await LuaWasm.initialize(this.customWasmUri, this.environmentVariables);
            }
            return this.lua;
        }
    }

    exports.Decoration = Decoration;
    exports.LUAI_MAXSTACK = LUAI_MAXSTACK;
    exports.LUA_MULTRET = LUA_MULTRET;
    exports.LUA_REGISTRYINDEX = LUA_REGISTRYINDEX;
    exports.LuaEngine = LuaEngine;
    exports.LuaFactory = LuaFactory;
    exports.LuaGlobal = Global;
    exports.LuaMultiReturn = MultiReturn;
    exports.LuaRawResult = RawResult;
    exports.LuaThread = Thread;
    exports.LuaTimeoutError = LuaTimeoutError;
    exports.LuaTypeExtension = LuaTypeExtension;
    exports.LuaWasm = LuaWasm;
    exports.PointerSize = PointerSize;
    exports.decorate = decorate;
    exports.decorateFunction = decorateFunction;
    exports.decorateProxy = decorateProxy;
    exports.decorateUserdata = decorateUserdata;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
